# -*- coding: utf-8 -*-
"""Windows preprocessor plugins."""

import os

from plaso.containers import artifacts
from plaso.lib import dtfabric_helper
from plaso.lib import errors
from plaso.helpers.windows import languages
from plaso.preprocessors import interface
from plaso.preprocessors import manager


class WindowsEnvironmentVariableArtifactPreprocessorPlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """Windows environment variable artifact preprocessor plugin."""

  _NAME = None

  def _ParseValueData(self, mediator, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, str):
      raise errors.PreProcessFail(
          'Unsupported Windows Registry value type: {0!s} for '
          'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=self._NAME, value=value_data)

    try:
      mediator.AddEnvironmentVariable(environment_variable)
    except KeyError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to set environment variable: {0:s}.'.format(self._NAME))


class WindowsProfilePathEnvironmentVariableArtifactPreprocessorPlugin(
    interface.WindowsRegistryKeyArtifactPreprocessorPlugin):
  """Windows profile path environment variable artifact preprocessor plugin."""

  _NAME = None

  def _ParseKey(self, mediator, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value or None if not
          specified.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    try:
      registry_value = registry_key.GetValueByName('ProfilesDirectory')
    except IOError as exception:
      raise errors.PreProcessFail((
          'Unable to retrieve Windows Registry key: {0:s} value: '
          'ProfilesDirectory with error: {1!s}').format(
              registry_key.path, exception))

    profiles_directory = ''
    if registry_value:
      value_data = registry_value.GetDataAsObject()
      if isinstance(value_data, str):
        profiles_directory = value_data

    try:
      registry_value = registry_key.GetValueByName(value_name)
    except IOError as exception:
      raise errors.PreProcessFail((
          'Unable to retrieve Windows Registry key: {0:s} value: {1:s} '
          'with error: {2!s}').format(
              registry_key.path, value_name, exception))

    if not registry_value:
      return

    value_data = registry_value.GetDataAsObject()
    if not isinstance(value_data, str):
      raise errors.PreProcessFail(
          'Unsupported Windows Registry value type: {0!s} for '
          'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    first_path_segment = value_data.split('\\')[0]

    # If the first path segment does not starts with an environment variable or
    # is absolute, consider it to be a relative path and prefix it with the
    # ProfilesDirectory value.
    if first_path_segment[0] == '%' and first_path_segment[-1] == '%':
      profile_path = value_data
    elif not first_path_segment or first_path_segment[1:2] == ':\\':
      profile_path = value_data
    else:
      profile_path = '\\'.join([profiles_directory.rstrip('\\'), value_data])

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=self._NAME, value=profile_path)

    try:
      mediator.AddEnvironmentVariable(environment_variable)
    except KeyError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to set environment variable: {0:s}.'.format(self._NAME))


class WindowsPathEnvironmentVariableArtifactPreprocessorPlugin(
    interface.FileSystemArtifactPreprocessorPlugin):
  """Windows path environment variable plugin interface."""

  _NAME = None

  def _ParsePathSpecification(
      self, mediator, searcher, file_system, path_specification,
      path_separator):
    """Parses artifact file system data for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      searcher (dfvfs.FileSystemSearcher): file system searcher to preprocess
          the file system.
      file_system (dfvfs.FileSystem): file system to be preprocessed.
      path_specification (dfvfs.PathSpec): path specification that contains
          the artifact value data.
      path_separator (str): path segment separator.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    relative_path = searcher.GetRelativePath(path_specification)
    if not relative_path:
      raise errors.PreProcessFail(
          'Unable to read: {0:s} with error: missing relative path'.format(
              self.ARTIFACT_DEFINITION_NAME))

    if path_separator != file_system.PATH_SEPARATOR:
      relative_path_segments = file_system.SplitPath(relative_path)
      relative_path = '{0:s}{1:s}'.format(
          path_separator, path_separator.join(relative_path_segments))

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=self._NAME, value=relative_path)

    try:
      mediator.AddEnvironmentVariable(environment_variable)
    except KeyError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to set environment variable: {0:s}.'.format(self._NAME))


class WindowsAllUsersAppDataKnowledgeBasePlugin(
    interface.KnowledgeBasePreprocessorPlugin):
  """The allusersdata knowledge base value plugin.

  The allusersdata value is needed for the expansion of
  %%environ_allusersappdata%% in artifact definitions.
  """

  def Collect(self, mediator):
    """Collects values from the knowledge base.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.

    Raises:
      PreProcessFail: if the preprocessing fails.
    """
    environment_variable = mediator.GetEnvironmentVariable('programdata')
    allusersappdata = getattr(environment_variable, 'value', None)

    if not allusersappdata:
      environment_variable = mediator.GetEnvironmentVariable('allusersprofile')
      allusersdata = getattr(environment_variable, 'value', None)

      if allusersdata:
        allusersappdata = '\\'.join([allusersdata, 'Application Data'])

    if allusersappdata:
      environment_variable = artifacts.EnvironmentVariableArtifact(
          case_sensitive=False, name='allusersappdata', value=allusersappdata)

      try:
        mediator.AddEnvironmentVariable(environment_variable)
      except KeyError:
        mediator.ProducePreprocessingWarning(
            self.__class__.__name__,
            'Unable to set environment variable: %AllUsersAppData% in.')


class WindowsAllUsersProfileEnvironmentVariablePlugin(
    WindowsProfilePathEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %AllUsersProfile% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsEnvironmentVariableAllUsersProfile'

  _NAME = 'allusersprofile'


class WindowsAllUsersAppProfileKnowledgeBasePlugin(
    interface.KnowledgeBasePreprocessorPlugin):
  """The allusersprofile knowledge base value plugin.

  The allusersprofile value is needed for the expansion of
  %%environ_allusersappprofile%% in artifact definitions.

  It is derived from %ProgramData% for versions of Windows, Vista and later,
  that do not define %AllUsersProfile%.
  """

  def Collect(self, mediator):
    """Collects values from the knowledge base.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.

    Raises:
      PreProcessFail: if the preprocessing fails.
    """
    environment_variable = mediator.GetEnvironmentVariable('allusersprofile')
    allusersprofile = getattr(environment_variable, 'value', None)

    if not allusersprofile:
      environment_variable = mediator.GetEnvironmentVariable('programdata')
      allusersprofile = getattr(environment_variable, 'value', None)

      if allusersprofile:
        environment_variable = artifacts.EnvironmentVariableArtifact(
            case_sensitive=False, name='allusersprofile', value=allusersprofile)

        try:
          mediator.AddEnvironmentVariable(environment_variable)
        except KeyError:
          mediator.ProducePreprocessingWarning(
              self.__class__.__name__,
              'Unable to set environment variable: %AllUsersProfile% in.')


class WindowsAvailableTimeZonesPlugin(
    interface.WindowsRegistryKeyArtifactPreprocessorPlugin,
    dtfabric_helper.DtFabricHelper):
  """The Windows available time zones plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsAvailableTimeZones'

  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'time_zone_information.yaml')

  def _ParseKey(self, mediator, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value or None if not
          specified.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    std_value = registry_key.GetValueByName('Std')
    if std_value:
      localized_name = std_value.GetDataAsObject()
    else:
      localized_name = registry_key.name

    mui_std_value = registry_key.GetValueByName('MUI_Std')
    if mui_std_value:
      mui_form = mui_std_value.GetDataAsObject()
    else:
      mui_form = None

    tzi_value = registry_key.GetValueByName('TZI')
    if not tzi_value:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'TZI value missing from Windows Registry key: {0:s}'.format(
              registry_key.key_path))
      return

    time_zone_artifact = artifacts.TimeZoneArtifact(
        localized_name=localized_name, mui_form=mui_form,
        name=registry_key.name)

    try:
      self._ParseTZIValue(tzi_value.data, time_zone_artifact)

    except (ValueError, errors.ParseError) as exception:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to parse TZI record value in Windows Registry key: {0:s} '
          'with error: {2!s}'.format(
              registry_key.key_path, exception))
      return

    try:
      mediator.AddTimeZoneInformation(time_zone_artifact)
    except KeyError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          ('Unable to add time zone information: {0:s} to knowledge '
           'base.').format(registry_key.name))

  def _ParseTZIValue(self, value_data, time_zone_artifact):
    """Parses the time zone information (TZI) value data.

    Args:
      value_data (bytes): time zone information (TZI) value data.
      time_zone_artifact (TimeZoneArtifact): time zone artifact.

    Raises:
      ParseError: if the value data could not be parsed.
    """
    data_type_map = self._GetDataTypeMap('tzi_record')

    tzi_record = self._ReadStructureFromByteStream(
        value_data, 0, data_type_map)

    if tzi_record.standard_bias:
      time_zone_artifact.offset = tzi_record.standard_bias
    else:
      time_zone_artifact.offset = tzi_record.bias


class WindowsCodePagePlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows code page plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsCodePage'

  def _ParseValueData(self, mediator, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, str):
      raise errors.PreProcessFail(
          'Unsupported Windows Registry value type: {0!s} for '
          'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    # Map the Windows code page name to a Python equivalent name.
    code_page = 'cp{0:s}'.format(value_data)

    try:
      mediator.SetCodePage(code_page)
    except ValueError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME, 'Unable to set code page.')


class WindowsEventLogPublishersPlugin(
    interface.WindowsRegistryKeyArtifactPreprocessorPlugin):
  """The Windows EventLog publishers plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsEventLogPublishers'

  def _ParseKey(self, mediator, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value or None if not
          specified.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    registry_value = registry_key.GetValueByName('')
    if not registry_value:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'EventLog source missing for: {0:s}'.format(
              registry_key.name))
      return

    log_source = registry_value.GetDataAsObject()

    event_message_files = None
    registry_value = registry_key.GetValueByName('MessageFileName')
    if registry_value:
      event_message_files = registry_value.GetDataAsObject()
      event_message_files = sorted(filter(None, [
          path.strip().lower() for path in event_message_files.split(';')]))

    provider_identifier = registry_key.name.lower()

    if log_source:
      log_source = log_source.lower()

    windows_event_log_provider = artifacts.WindowsEventLogProviderArtifact(
        event_message_files=event_message_files, identifier=provider_identifier,
        log_source=log_source)

    try:
      mediator.AddWindowsEventLogProvider(windows_event_log_provider)
    except KeyError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to set add Windows EventLog provider: {0:s}.',format(
              log_source))


class WindowsEventLogSourcesPlugin(
    interface.WindowsRegistryKeyArtifactPreprocessorPlugin):
  """The Windows EventLog sources plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsEventLogSources'

  def _ParseKey(self, mediator, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value or None if not
          specified.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    category_message_files = None
    registry_value = registry_key.GetValueByName('CategoryMessageFile')
    if registry_value:
      category_message_files = registry_value.GetDataAsObject()
      category_message_files = sorted(filter(None, [
          path.strip().lower() for path in category_message_files.split(';')]))

    event_message_files = None
    registry_value = registry_key.GetValueByName('EventMessageFile')
    if registry_value:
      event_message_files = registry_value.GetDataAsObject()
      event_message_files = sorted(filter(None, [
          path.strip().lower() for path in event_message_files.split(';')]))

    parameter_message_files = None
    registry_value = registry_key.GetValueByName('ParameterMessageFile')
    if registry_value:
      parameter_message_files = registry_value.GetDataAsObject()
      parameter_message_files = sorted(filter(None, [
          path.strip().lower() for path in parameter_message_files.split(';')]))

    provider_identifier = None
    registry_value = registry_key.GetValueByName('ProviderGuid')
    if registry_value:
      provider_identifier = registry_value.GetDataAsObject()
      provider_identifier = provider_identifier.lower()

    key_path_segments = registry_key.path.split('\\')

    log_source = key_path_segments[-1]
    if log_source:
      log_source = log_source.lower()

    log_type = key_path_segments[-2]
    if log_type:
      log_type = log_type.lower()

    windows_event_log_provider = artifacts.WindowsEventLogProviderArtifact(
        category_message_files=category_message_files,
        event_message_files=event_message_files, identifier=provider_identifier,
        log_source=log_source, log_type=log_type,
        parameter_message_files=parameter_message_files)

    try:
      mediator.AddWindowsEventLogProvider(windows_event_log_provider)
    except KeyError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to set add Windows EventLog provider: {0:s}/{1:s}.'.format(
              log_type, log_source))


class WindowsHostnamePlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows hostname plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsComputerName'

  def _ParseValueData(self, mediator, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, str):
      if not hasattr(value_data, '__iter__'):
        raise errors.PreProcessFail(
            'Unsupported Windows Registry value type: {0!s} for '
            'artifact: {1:s}.'.format(
                type(value_data), self.ARTIFACT_DEFINITION_NAME))

      # If the value data is a multi string only use the first string.
      value_data = value_data[0]

    hostname_artifact = artifacts.HostnameArtifact(name=value_data)
    mediator.AddHostname(hostname_artifact)


class WindowsLanguagePlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows language plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsLanguage'

  def _ParseValueData(self, mediator, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, str):
      raise errors.PreProcessFail(
          'Unsupported Windows Registry value type: {0!s} for '
          'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    try:
      lcid = int(value_data, 16)
      language_tag = languages.WindowsLanguageHelper.GetLanguageTagForLCID(lcid)
    except ValueError:
      language_tag = None

    if language_tag:
      mediator.SetLanguage(language_tag)
    else:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to determine language tag for LCID: {0:s}.'.format(
              value_data))


class WindowsMountedDevicesPlugin(
    interface.WindowsRegistryKeyArtifactPreprocessorPlugin,
    dtfabric_helper.DtFabricHelper):
  """The Windows mounted devices plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsMountedDevices'

  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'mounted_devices.yaml')

  def _ParseKey(self, mediator, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value or None if not
          specified.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    for registry_value in registry_key.GetValues():
      mounted_device_artifact = artifacts.WindowsMountedDeviceArtifact(
          identifier=registry_value.name)

      # TODO: parse registry_value.data
      value_data_size = len(registry_value.data)
      if value_data_size == 12:
        data_type_map = self._GetDataTypeMap('mounted_devices_mbr_partition')

        try:
          partition_values = self._ReadStructureFromByteStream(
              registry_value.data, 0, data_type_map)

          mounted_device_artifact.disk_identity = partition_values.disk_identity
          mounted_device_artifact.partition_offset = (
              partition_values.partition_offset)

        except (ValueError, errors.ParseError) as exception:
          mediator.ProducePreprocessingWarning(
              self.ARTIFACT_DEFINITION_NAME,
              ('Unable to parse mounted devices MBR partition value Windows '
               'Registry value: {0:s} with error: {1!s}').format(
                  registry_value.name, exception))

      elif value_data_size == 24:
        data_type_map = self._GetDataTypeMap('mounted_devices_gpt_partition')

        try:
          partition_values = self._ReadStructureFromByteStream(
              registry_value.data, 0, data_type_map)

          mounted_device_artifact.partition_identifier = str(
              partition_values.partition_identifier)

        except (ValueError, errors.ParseError) as exception:
          mediator.ProducePreprocessingWarning(
              self.ARTIFACT_DEFINITION_NAME,
              ('Unable to parse mounted devices GPT partition value Windows '
               'Registry value: {0:s} with error: {1!s}').format(
                  registry_value.name, exception))

      else:
        try:
          mounted_device_artifact.device = registry_value.data.decode(
              'utf-16-le')
        except UnicodeDecodeError as exception:
          mediator.ProducePreprocessingWarning(
              self.ARTIFACT_DEFINITION_NAME,
              ('Unable to parse mounted devices device string value Windows '
               'Registry value: {0:s} with error: {1!s}').format(
                  registry_value.name, exception))

      try:
        mediator.AddArtifact(mounted_device_artifact)
      except KeyError:
        mediator.ProducePreprocessingWarning(
            self.ARTIFACT_DEFINITION_NAME,
            'Unable to add Windows mounted device: {0:s} artifact.'.format(
                registry_value.name))


class WindowsProgramDataEnvironmentVariablePlugin(
    WindowsProfilePathEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %ProgramData% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsEnvironmentVariableProgramData'

  _NAME = 'programdata'


class WindowsProgramDataKnowledgeBasePlugin(
    interface.KnowledgeBasePreprocessorPlugin):
  """The programdata knowledge base value plugin.

  The programdata value is needed for the expansion of %%environ_programdata%%
  in artifact definitions.

  It is derived from %AllUsersProfile% for versions of Windows prior to Vista
  that do not define %ProgramData%.
  """

  def Collect(self, mediator):
    """Collects values from the knowledge base.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.

    Raises:
      PreProcessFail: if the preprocessing fails.
    """
    environment_variable = mediator.GetEnvironmentVariable('programdata')
    allusersprofile = getattr(environment_variable, 'value', None)

    if not allusersprofile:
      environment_variable = mediator.GetEnvironmentVariable('allusersprofile')
      allusersprofile = getattr(environment_variable, 'value', None)

      if allusersprofile:
        environment_variable = artifacts.EnvironmentVariableArtifact(
            case_sensitive=False, name='programdata', value=allusersprofile)

        try:
          mediator.AddEnvironmentVariable(environment_variable)
        except KeyError:
          mediator.ProducePreprocessingWarning(
              self.__class__.__name__,
              'Unable to set environment variable: %ProgramData%.')


class WindowsProgramFilesEnvironmentVariablePlugin(
    WindowsEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %ProgramFiles% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsEnvironmentVariableProgramFiles'

  _NAME = 'programfiles'


class WindowsProgramFilesX86EnvironmentVariablePlugin(
    WindowsEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %ProgramFilesX86% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsEnvironmentVariableProgramFilesX86'

  _NAME = 'programfilesx86'


class WindowsServicesAndDriversPlugin(
    interface.WindowsRegistryKeyArtifactPreprocessorPlugin):
  """The Windows service (and driver) configurations plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsServices'

  def _ParseKey(self, mediator, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value or None if not
          specified.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    service_type = None
    start_type = None

    registry_value = registry_key.GetValueByName('Type')
    if registry_value:
      service_type = registry_value.GetDataAsObject()

    registry_value = registry_key.GetValueByName('Start')
    if registry_value:
      start_type = registry_value.GetDataAsObject()

    if None in (service_type, start_type):
      return

    service_configuration = artifacts.WindowsServiceConfigurationArtifact(
        name=registry_key.name, service_type=service_type,
        start_type=start_type)

    registry_value = registry_key.GetValueByName('ErrorControl')
    if registry_value:
      service_configuration.error_control = registry_value.GetDataAsObject()

    registry_value = registry_key.GetValueByName('ImagePath')
    if registry_value:
      service_configuration.image_path = registry_value.GetDataAsObject()

    registry_value = registry_key.GetValueByName('ObjectName')
    if registry_value:
      service_configuration.object_name = registry_value.GetDataAsObject()

    sub_registry_key = registry_key.GetSubkeyByName('Parameters')
    if sub_registry_key:
      registry_value = sub_registry_key.GetValueByName('ServiceDll')
      if registry_value:
        service_configuration.service_dll = registry_value.GetDataAsObject()

    try:
      mediator.AddArtifact(service_configuration)
    except KeyError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to add Windows service configuation: {0:s} artifact.'.format(
              registry_value.name))


class WindowsSystemProductPlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows system product information plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsProductName'

  def _ParseValueData(self, mediator, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, str):
      raise errors.PreProcessFail(
          'Unsupported Windows Registry value type: {0!s} for '
          'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    mediator.SetValue('operating_system_product', value_data)


class WindowsSystemRootEnvironmentVariablePlugin(
    WindowsPathEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %SystemRoot% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsEnvironmentVariableSystemRoot'

  _NAME = 'systemroot'


class WindowsSystemVersionPlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows system version information plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsCurrentVersion'

  def _ParseValueData(self, mediator, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, str):
      raise errors.PreProcessFail(
          'Unsupported Windows Registry value type: {0!s} for '
          'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    mediator.SetValue('operating_system_version', value_data)


class WindowsTimeZonePlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows time zone plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsTimezone'

  def _ParseValueData(self, mediator, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, str):
      raise errors.PreProcessFail(
          'Unsupported Windows Registry value type: {0!s} for '
          'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    try:
      mediator.SetTimeZone(value_data)
    except ValueError as execption:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          'Unable to map: "{0:s}" to time zone with error: {1!s}'.format(
              value_data, execption))


class WindowsUserAccountsPlugin(
    interface.WindowsRegistryKeyArtifactPreprocessorPlugin):
  """The Windows user account plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsRegistryProfiles'

  def _GetUsernameFromProfilePath(self, path):
    """Retrieves the username from a Windows profile path.

    Trailing path path segment are ignored.

    Args:
      path (str): a Windows path with '\\' as path segment separator.

    Returns:
      str: basename which is the last path segment.
    """
    # Strip trailing key separators.
    while path and path[-1] == '\\':
      path = path[:-1]

    if path:
      _, _, path = path.rpartition('\\')
    return path

  def _ParseKey(self, mediator, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      mediator (PreprocessMediator): mediates interactions between preprocess
          plugins and other components, such as storage.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value or None if not
          specified.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    user_account = artifacts.UserAccountArtifact(
        identifier=registry_key.name, path_separator='\\')

    registry_value = registry_key.GetValueByName('ProfileImagePath')
    if registry_value:
      profile_path = registry_value.GetDataAsObject()
      username = self._GetUsernameFromProfilePath(profile_path)

      user_account.user_directory = profile_path or None
      user_account.username = username or None

    try:
      mediator.AddUserAccount(user_account)
    except KeyError:
      mediator.ProducePreprocessingWarning(
          self.ARTIFACT_DEFINITION_NAME,
          ('Unable to add user account: "{0!s}" to knowledge '
           'base').format(username))


class WindowsWinDirEnvironmentVariablePlugin(
    WindowsPathEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %WinDir% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = 'WindowsEnvironmentVariableWinDir'

  _NAME = 'windir'


manager.PreprocessPluginsManager.RegisterPlugins([
    WindowsAllUsersAppDataKnowledgeBasePlugin,
    WindowsAllUsersProfileEnvironmentVariablePlugin,
    WindowsAllUsersAppProfileKnowledgeBasePlugin,
    WindowsAvailableTimeZonesPlugin,
    WindowsCodePagePlugin, WindowsEventLogPublishersPlugin,
    WindowsEventLogSourcesPlugin, WindowsHostnamePlugin, WindowsLanguagePlugin,
    WindowsMountedDevicesPlugin, WindowsProgramDataEnvironmentVariablePlugin,
    WindowsProgramDataKnowledgeBasePlugin,
    WindowsProgramFilesEnvironmentVariablePlugin,
    WindowsProgramFilesX86EnvironmentVariablePlugin,
    WindowsServicesAndDriversPlugin, WindowsSystemProductPlugin,
    WindowsSystemRootEnvironmentVariablePlugin, WindowsSystemVersionPlugin,
    WindowsTimeZonePlugin, WindowsWinDirEnvironmentVariablePlugin,
    WindowsUserAccountsPlugin])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Apple iOS Car Play application plist plugin."""

import unittest

from plaso.parsers.plist_plugins import ios_carplay

from tests.parsers.plist_plugins import test_lib


class IOSCarPlayPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Apple iOS Car Play application plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'com.apple.CarPlayApp.plist'

    plugin = ios_carplay.IOSCarPlayPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 5)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'application_identifier': 'com.apple.mobilecal',
        'data_type': 'ios:carplay:history:entry',
        'last_run_time': '2020-04-12T13:55:51.255235072+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugin for WinCC log files."""

from dfvfs.helpers import text_file

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import interface
from plaso.parsers import manager


class SIMATICS7EventData(events.EventData):
  """SIMATIC S7 event data.

  Attributes:
    body (str): the message content of the event.
    creation_time (dfdatetime.DateTimeValues): date and time the log entry
        was created.
  """

  DATA_TYPE = 'wincc:simatic_s7:entry'

  def __init__(self):
    """Initializes event data."""
    super(SIMATICS7EventData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None
    self.creation_time = None


class WinCCSysLogEventData(events.EventData):
  """WinCC Sys Log event data.

  Attributes:
    body (str): the content of the log's message.
    creation_time (dfdatetime.DateTimeValues): date and time the log entry
        was created.
    event_number (int): a number specifying the type of event.
    log_hostname (str): the hostname of the machine logging the event.
    log_identifier (int): identifier for this log file.
    source_device (str): which device generated the event.
  """

  DATA_TYPE = 'wincc:sys_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(WinCCSysLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None
    self.creation_time = None
    self.event_number = None
    self.log_hostname = None
    self.log_identifier = None
    self.source_device = None


class SIMATICLogParser(interface.FileObjectParser):
  """Text parser plugin for SIMATIC S7 Log files."""

  NAME = 'simatic_s7'
  DATA_FORMAT = 'SIMATIC S7 Log file'

  DELIMITER = ','
  ENCODING = 'ascii'
  END_OF_LINE = '\r\n'

  _EXPECTED_FIRST_LINE_STRING = 'Log starting ...' + END_OF_LINE
  _EXPECTED_SECOND_LINE_STRING = '| LogFileName'
  _EXPECTED_THIRD_LINE_STRING = '| LogFileCount'

  def _ParseValues(self, parser_mediator, line_number, values):
    """Parses SIMATIC S7 log file values.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      line_number (int): number of the line the values were extracted from.
      values (list[str]): values extracted from the line.

    Raises:
      WrongParser: when the values cannot be parsed.
    """
    number_of_values = len(values)
    if number_of_values < 2:
      error_string = 'Expected at least two values on line {0:d}'.format(
          line_number)
      raise errors.WrongParser(error_string)

    # The first lines seem to  always follow the following format:
    #
    # 2019-05-27 10:05:43,405 INFO     Log starting ...
    # 2019-05-27 10:05:43,405 INFO     | LogFileName   : C:\.....
    # 2019-05-27 10:05:43,419 INFO     | LogFileCount  : 3
    if line_number == 0:
      if not values[1].endswith(self._EXPECTED_FIRST_LINE_STRING):
        error_string = 'Expected first line to end with "{0:s}"."{1!s}"'.format(
            self._EXPECTED_FIRST_LINE_STRING, values[1])
        raise errors.WrongParser(error_string)

    if line_number == 1:
      if values[1].find(self._EXPECTED_SECOND_LINE_STRING) < 0:
        error_string = 'Expected second line to contain "{0:s}"'.format(
            self._EXPECTED_SECOND_LINE_STRING)
        raise errors.WrongParser(error_string)

    if line_number == 2:
      if values[1].find(self._EXPECTED_THIRD_LINE_STRING) < 0:
        error_string = 'Expected third line to contain {0:s}'.format(
            self._EXPECTED_THIRD_LINE_STRING)
        raise errors.WrongParser(error_string)

    event_data = SIMATICS7EventData()
    try:
      date_time = dfdatetime_time_elements.TimeElements()
      date_time.CopyFromDateTimeString(values[0])
      event_data.creation_time = date_time
    except (TypeError, ValueError) as exception:
      error_string = (
          'Unable to parse date time value with error: '
          '{0!s} on line {1:d} {2!s}').format(exception, line_number, values)
      parser_mediator.ProduceExtractionWarning(error_string)

    event_data.body = ','.join(values[1:]).strip()

    parser_mediator.ProduceEventData(event_data)

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a SIMATIC Log file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      file_object (dfvfs.FileIO): file-like object.

    Raises:
      WrongParser: when the file cannot be parsed.
    """
    line_reader = text_file.TextFile(
        file_object, encoding=self.ENCODING, end_of_line=self.END_OF_LINE)

    line_number = 0

    try:
      line = line_reader.readline()
    except UnicodeDecodeError as exception:
      raise errors.WrongParser(
          'unable to read line: {0:d} with error: {1!s}'.format(
              line_number, exception))

    while line:
      values = line.split(self.DELIMITER)
      self._ParseValues(parser_mediator, line_number, values)

      try:
        line_number += 1
        line = line_reader.readline()
      except UnicodeDecodeError as exception:
        parser_mediator.ProduceExtractionWarning(
            'unable to read line: {0:d} with error: {1!s}'.format(
                line_number, exception))
        break


class WinCCSysLogParser(interface.FileObjectParser):
  """Text parser plugin for WinCC Sys Log files."""

  NAME = 'wincc_sys'
  DATA_FORMAT = 'WinCC Sys Log file'

  DELIMITER = ','
  ENCODING = 'utf-16-le'

  _DISALLOWED_HOSTNAME_CHARS = ['\\', '/', ':', '*', '?', '"', '<', '>', '|']

  _END_OF_LOG_FILE_STRING = '======>'

  def _ParseValues(self, parser_mediator, line_number, values, first_line):
    """Parses WinCC log file values.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      line_number (int): number of the line the values were extracted from.
      values (list[str]): values extracted from the line.
      first_line (bool): True if this is first line from which values were
          extracted.

    Raises:
      WrongParser: when the values cannot be parsed.
    """
    number_of_values = len(values)
    if len(values) < 10:
      error_string = 'invalid number of values : {0:d} in line: {1:d}'.format(
          number_of_values, line_number)

      # On other lines, we might encounter times where the split() operation
      # produces more values.
      if first_line:
        raise errors.WrongParser(error_string)

      parser_mediator.ProduceExtractionWarning(error_string)

    event_data = WinCCSysLogEventData()

    try:
      log_identifier = int(values[0], 10)
      event_data.log_identifier = log_identifier
    except ValueError as exception:
      error_string = (
          'Type of first value ({0!s}) should be an int in line: {1:d}').format(
              values[0], line_number)
      self._ParseValuesFail(
          parser_mediator, first_line, exception, error_string)

    try:
      date_string = values[1]
      time_string = values[2]
      day_of_month, month, year = [
          int(element, 10) for element in date_string.split('.')]
      hours, minutes, seconds, milliseconds = [
          int(element, 10) for element in time_string.split(':')]
      time_elements_tuple = (
          year, month, day_of_month, hours, minutes, seconds, milliseconds)
      date_time = dfdatetime_time_elements.TimeElementsInMilliseconds(
          time_elements_tuple=time_elements_tuple,
      )
      # Unsure about whether the format is dependant on the system's Locale
      date_time.is_local_time = True

      event_data.creation_time = date_time

    except (TypeError, ValueError) as exception:
      error_string = (
          'Unable to parse time elements with error: '
          '{0!s} on line {1:d} {2!s}').format(
              exception, line_number, values[1:2])
      self._ParseValuesFail(
          parser_mediator, first_line, exception, error_string)

    try:
      event_data.event_number = int(values[3], 10)
    except ValueError as exception:
      error_string = (
          'Type of event_number value ({0!s}) should be a decimal in line:'
          '{1:d}').format(values[3], line_number)
      self._ParseValuesFail(
          parser_mediator, first_line, exception, error_string)

    # Column 4 and 5 contain unknown values, we are not parsing them.

    # We are checking this only once, on the first line, as we expect all
    # following lines to contain the same hostname, as the logs are
    # collected from the system generating it.
    # Using this documentation to validate a Windows host name.
    # https://learn.microsoft.com/en-us/troubleshoot/windows-server/identity/naming-conventions-for-computer-domain-site-ou
    hostname = values[6]
    if first_line:
      if not hostname:
        error_string = (
            'Hostname ({0!s}) needs to be at least 1 character on line:'
            '{1:d}').format(hostname, line_number)
        self._ParseValuesFail(
            parser_mediator, first_line, exception, error_string)
      if len(hostname) > 16:
        error_string = (
            'Hostname ({0!s}) can\'t be longer than 15 characters on line:'
            '{1:d}').format(hostname, line_number)
        self._ParseValuesFail(
            parser_mediator, first_line, exception, error_string)
      for character in self._DISALLOWED_HOSTNAME_CHARS:
        if character in hostname:
          error_string = (
              'Hostname ({0!s}) can\'t contain the character {1:s} on line'
              '{2:d}').format(hostname, character, line_number)
          self._ParseValuesFail(
              parser_mediator, first_line, exception, error_string)

    event_data.log_hostname = hostname

    event_data.source_device = values[7]

    # The second to last field is the one that might contain unquoted separator.
    # The last field can contain a string such as MSG_STATE_COME, MSG_STATE_GO,
    # but also sometimes contains a message string.

    text_message = ' '.join(values[8:])
    event_data.body = text_message

    parser_mediator.ProduceEventData(event_data)

  def _ParseValuesFail(
      self, parser_mediator, first_line, parent_exception, error_string):
    if first_line:
      raise errors.WrongParser(error_string) from parent_exception

    parser_mediator.ProduceExtractionWarning(error_string)

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a WinCC Sys Log file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      file_object (dfvfs.FileIO): file-like object.

    Raises:
      WrongParser: when the file cannot be parsed.
    """
    # Note that we cannot use the DSVParser here since this WinCC file format is
    # not strict and clean file format.
    line_reader = text_file.TextFile(
        file_object, encoding=self.ENCODING, end_of_line='\r\n')

    line_number = 0
    first_line = True

    # While the file is in UTF-16, the end_of_line character is '\r\n'.
    # We manually strip this out.

    try:
      line = line_reader.readline().strip()
    except UnicodeDecodeError as exception:
      raise errors.WrongParser(
          'unable to read line: {0:d} with error: {1!s}'.format(
              line_number, exception))

    while line:
      if line.startswith(self._END_OF_LOG_FILE_STRING):
        # It seems that sometimes WinCC logs will end with a line starting like
        # this, with the name of the next log file.
        # But this is not always the case though.
        line = line_reader.readline().strip()
        continue
      values = line.split(self.DELIMITER)
      self._ParseValues(parser_mediator, line_number, values, first_line)

      try:
        line_number += 1
        first_line = False
        line = line_reader.readline().strip()
      except UnicodeDecodeError as exception:
        parser_mediator.ProduceExtractionWarning(
            'unable to read line: {0:d} with error: {1!s}'.format(
                line_number, exception))
        break


manager.ParsersManager.RegisterParsers([WinCCSysLogParser, SIMATICLogParser])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the heaps to sort events in chronological order."""

import unittest

from plaso.lib import definitions
from plaso.storage.fake import event_heap

from tests.containers import test_lib as containers_test_lib
from tests.storage import test_lib


class EventHeapTest(test_lib.StorageTestCase):
  """Tests for the event heap."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'timestamp': 5134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_WRITTEN},
      {'data_type': 'test:event',
       'timestamp': 2345871286,
       'timestamp_desc': definitions.TIME_DESCRIPTION_WRITTEN}]

  def testNumberOfEvents(self):
    """Tests the number_of_events property."""
    test_heap = event_heap.EventHeap()
    self.assertEqual(test_heap.number_of_events, 0)

  def testPopEvent(self):
    """Tests the PopEvent function."""
    test_heap = event_heap.EventHeap()

    self.assertEqual(len(test_heap._heap), 0)

    test_event = test_heap.PopEvent()
    self.assertIsNone(test_event)

    event_index = 0
    for event, _, _ in containers_test_lib.CreateEventsFromValues(
        self._TEST_EVENTS):
      test_heap.PushEvent(event, event_index)
      event_index += 1

    self.assertEqual(len(test_heap._heap), 2)

    test_event = test_heap.PopEvent()
    self.assertIsNotNone(test_event)

    self.assertEqual(len(test_heap._heap), 1)

  def testPopEvents(self):
    """Tests the PopEvents function."""
    test_heap = event_heap.EventHeap()

    self.assertEqual(len(test_heap._heap), 0)

    test_events = list(test_heap.PopEvents())
    self.assertEqual(len(test_events), 0)

    event_index = 0
    for event, _, _ in containers_test_lib.CreateEventsFromValues(
        self._TEST_EVENTS):
      test_heap.PushEvent(event, event_index)
      event_index += 1

    self.assertEqual(len(test_heap._heap), 2)

    test_events = list(test_heap.PopEvents())
    self.assertEqual(len(test_events), 2)

    self.assertEqual(len(test_heap._heap), 0)

  def testPushEvent(self):
    """Tests the PushEvent function."""
    test_heap = event_heap.EventHeap()

    self.assertEqual(len(test_heap._heap), 0)

    event, _, _ = containers_test_lib.CreateEventFromValues(
        self._TEST_EVENTS[0])
    test_heap.PushEvent(event, 0)

    self.assertEqual(len(test_heap._heap), 1)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows Event Log resources database reader."""

import unittest

from plaso.output import winevt_rc

from tests import test_lib as shared_test_lib


class WinevtResourcesSqlite3DatabaseReaderTest(shared_test_lib.BaseTestCase):
  """Windows EventLog resources SQLite database reader."""

  def testGetMessage(self):
    """Tests the GetMessage function."""
    database_path = self._GetTestFilePath(['winevt-rc.db'])
    self._SkipIfPathNotExists(database_path)

    database_reader = winevt_rc.WinevtResourcesSqlite3DatabaseReader()

    database_reader.Open(database_path)

    expected_message_string = (
        'Your computer has detected that the IP address {0:s} for the Network '
        'Card with network address {2:s} is already in use on the network. '
        'Your computer will automatically attempt to obtain a different '
        'address.')

    message_string = database_reader.GetMessage(
        'Microsoft-Windows-Dhcp-Client', 0x00000409, 0xb00003ed)
    self.assertEqual(message_string, expected_message_string)

    database_reader.Close()


class WinevtResourcesHelperTest(shared_test_lib.BaseTestCase):
  """Tests for the Windows EventLog resources helper."""

  # pylint: disable=protected-access

  def testGetWinevtRcDatabaseMessageString(self):
    """Tests the _GetWinevtRcDatabaseMessageString function."""
    database_path = self._GetTestFilePath(['winevt-rc.db'])
    self._SkipIfPathNotExists(database_path)

    test_helper = winevt_rc.WinevtResourcesHelper(
        None, shared_test_lib.TEST_DATA_PATH, 0x00000409)

    expected_message_string = (
        'Your computer has detected that the IP address {0:s} for the Network '
        'Card with network address {2:s} is already in use on the network. '
        'Your computer will automatically attempt to obtain a different '
        'address.')

    message_string = test_helper._GetWinevtRcDatabaseMessageString(
        'Microsoft-Windows-Dhcp-Client', 0xb00003ed)
    self.assertEqual(message_string, expected_message_string)

  def testGetMessageString(self):
    """Tests the GetMessageString function."""
    database_path = self._GetTestFilePath(['winevt-rc.db'])
    self._SkipIfPathNotExists(database_path)

    test_helper = winevt_rc.WinevtResourcesHelper(
        None, shared_test_lib.TEST_DATA_PATH, 0x00000409)

    expected_message_string = (
        'Your computer has detected that the IP address {0:s} for the Network '
        'Card with network address {2:s} is already in use on the network. '
        'Your computer will automatically attempt to obtain a different '
        'address.')

    message_string = test_helper.GetMessageString(
        '{15a7a4f8-0072-4eab-abad-f98a4d666aed}',
        'Microsoft-Windows-Dhcp-Client', 0xb00003ed, None)
    self.assertEqual(message_string, expected_message_string)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests the single process processing engine."""

import collections
import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import context

from plaso.engine import configurations
from plaso.single_process import extraction_engine
from plaso.storage.fake import writer as fake_writer

from tests import test_lib as shared_test_lib


class SingleProcessEngineTest(shared_test_lib.BaseTestCase):
  """Tests for the single process engine object."""

  # pylint: disable=protected-access

  def testProcessSource(self):
    """Tests the PreprocessSource and ProcessSource functions."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['ímynd.dd'])
    self._SkipIfPathNotExists(test_file_path)

    test_engine = extraction_engine.SingleProcessEngine()
    test_engine.BuildArtifactsRegistry(test_artifacts_path, None)
    resolver_context = context.Context()

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location='/',
        parent=os_path_spec)

    processing_configuration = configurations.ProcessingConfiguration()
    processing_configuration.data_location = shared_test_lib.DATA_PATH
    processing_configuration.parser_filter_expression = 'filestat'

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      system_configurations = test_engine.PreprocessSource(
          [source_path_spec], storage_writer)

      processing_status = test_engine.ProcessSource(
          storage_writer, resolver_context, processing_configuration,
          system_configurations, [source_path_spec])

      number_of_events = storage_writer.GetNumberOfAttributeContainers('event')
      number_of_extraction_warnings = (
          storage_writer.GetNumberOfAttributeContainers(
              'extraction_warning'))
      number_of_recovery_warnings = (
          storage_writer.GetNumberOfAttributeContainers(
              'recovery_warning'))

      parsers_counter = collections.Counter({
          parser_count.name: parser_count.number_of_events
          for parser_count in storage_writer.GetAttributeContainers(
              'parser_count')})

    finally:
      storage_writer.Close()

    self.assertFalse(processing_status.aborted)

    self.assertEqual(number_of_events, 15)
    self.assertEqual(number_of_extraction_warnings, 0)
    self.assertEqual(number_of_recovery_warnings, 0)

    expected_parsers_counter = collections.Counter({
        'filestat': 15,
        'total': 15})
    self.assertEqual(parsers_counter, expected_parsers_counter)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""A parser for Portable Executable (PE) files.

Also see:
  https://www.mandiant.com/resources/blog/tracking-malware-import-hashing
"""

import os

import pefile

from dfdatetime import posix_time as dfdatetime_posix_time
from dfvfs.helpers import data_slice as dfvfs_data_slice
from dtfabric.runtime import data_maps as dtfabric_data_maps

from plaso.containers import artifacts
from plaso.containers import events
from plaso.helpers.windows import languages
from plaso.helpers.windows import resource_files
from plaso.lib import errors
from plaso.lib import dtfabric_helper
from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import manager


class PEDLLImportEventData(events.EventData):
  """Portable Executable (PE) DLL import event data.

  Attributes:
    delayed_import (bool): True if the DLL is imported at run-time.
    modification_time (dfdatetime.DateTimeValues): last modification date and
        time.
    name (str): name of the imported DLL.
  """

  DATA_TYPE = 'pe_coff:dll_import'

  def __init__(self):
    """Initializes event data."""
    super(PEDLLImportEventData, self).__init__(data_type=self.DATA_TYPE)
    self.delayed_import = None
    self.modification_time = None
    self.name = None


class PEFileEventData(events.EventData):
  """Portable Executable (PE) file event data.

  Attributes:
    creation_time (dfdatetime.DateTimeValues): creation date and time.
    export_dll_name (str): name of the exported DLL.
    export_table_modification_time (dfdatetime.DateTimeValues): export table
        last modification date and time.
    imphash (str): "Import Hash" of the Portable Executable (PE) file.
    load_configuration_table_modification_time (dfdatetime.DateTimeValues):
        load configuration table last modification date and time.
    pe_type (str): type of Portable Executable (PE) file.
    section_names (list[str]): names of the sections in the Portable Executable
        (PE) file.
  """

  DATA_TYPE = 'pe_coff:file'

  def __init__(self):
    """Initializes event data."""
    super(PEFileEventData, self).__init__(data_type=self.DATA_TYPE)
    self.creation_time = None
    self.export_dll_name = None
    self.export_table_modification_time = None
    self.imphash = None
    self.load_configuration_table_modification_time = None
    self.pe_type = None
    self.section_names = None


class PEResourceEventData(events.EventData):
  """Portable Executable (PE) resource event data.

  Attributes:
    identifier (int): identifier of the resource.
    modification_time (dfdatetime.DateTimeValues): last modification date and
        time.
    name (str): name of the resource.
  """

  DATA_TYPE = 'pe_coff:resource'

  def __init__(self):
    """Initializes event data."""
    super(PEResourceEventData, self).__init__(data_type=self.DATA_TYPE)
    self.identifier = None
    self.modification_time = None
    self.name = None


class PEParser(interface.FileObjectParser, dtfabric_helper.DtFabricHelper):
  """Parser for Portable Executable (PE) files."""

  NAME = 'pe'
  DATA_FORMAT = 'Portable Executable (PE) file'

  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'pe_resources.yaml')

  _PE_DIRECTORIES = [
      pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT'],
      pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_EXPORT'],
      pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_IMPORT'],
      pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG'],
      pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_RESOURCE']]

  def __init__(self):
    """Initializes a PE parser."""
    super(PEParser, self).__init__()
    self._resource_file_helper = resource_files.WindowsResourceFileHelper

  def _GetPEType(self, pefile_object):
    """Retrieves the type of the PE file.

    Args:
      pefile_object (pefile.PE): pefile object.

    Returns:
      str: type of the Portable Executable (PE) file.
    """
    if pefile_object.is_dll():
      return 'Dynamic Link Library (DLL)'
    if pefile_object.is_exe():
      return 'Executable (EXE)'
    if pefile_object.is_driver():
      return 'Driver (SYS)'
    return 'Unknown PE type'

  def _GetSectionNames(self, pefile_object):
    """Retrieves all PE section names.

    Args:
      pefile_object (pefile.PE): pefile object.

    Returns:
      list[str]: names of the sections.
    """
    section_names = []
    for section in pefile_object.sections:
      section_name = getattr(section, 'Name', b'')
      # Ensure the name is decoded correctly.
      try:
        section_name = '{0:s}'.format(section_name.decode('unicode_escape'))
      except UnicodeDecodeError:
        section_name = '{0:s}'.format(repr(section_name))
      section_names.append(section_name)

    return section_names

  def _ParseLoadConfigurationTable(self, pefile_object, event_data):
    """Parses the load configuration table.

    Args:
      pefile_object (pefile.PE): pefile object.
      event_data (PEFileEventData): event data.
    """
    load_configuration_table = getattr(
        pefile_object, 'DIRECTORY_ENTRY_LOAD_CONFIG', None)
    if not load_configuration_table:
      return

    timestamp = getattr(load_configuration_table.struct, 'TimeDateStamp', None)
    if timestamp:
      date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
      event_data.load_configuration_table_modification_time = date_time

  def _ParseDelayImportTable(self, parser_mediator, pefile_object):
    """Parses the delay import table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      pefile_object (pefile.PE): pefile object.
    """
    delay_import_table = getattr(
        pefile_object, 'DIRECTORY_ENTRY_DELAY_IMPORT', None)
    if not delay_import_table:
      return

    for table_entry in delay_import_table:
      timestamp = getattr(table_entry.struct, 'dwTimeStamp', None)
      if not timestamp:
        continue

      dll_name = getattr(table_entry, 'dll', '')
      try:
        dll_name = dll_name.decode('ascii')
      except UnicodeDecodeError:
        dll_name = dll_name.decode('ascii', errors='replace')

      event_data = PEDLLImportEventData()
      event_data.delayed_import = True
      event_data.name = dll_name or None

      date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
      event_data.modification_time = date_time

      parser_mediator.ProduceEventData(event_data)

  def _ParseExportTable(self, pefile_object, event_data):
    """Parses the export table.

    Args:
      pefile_object (pefile.PE): pefile object.
      event_data (PEFileEventData): event data.
    """
    export_table = getattr(pefile_object, 'DIRECTORY_ENTRY_EXPORT', None)
    if not export_table:
      return

    dll_name = getattr(export_table, 'name', '')
    try:
      dll_name = dll_name.decode('ascii')
    except UnicodeDecodeError:
      dll_name = dll_name.decode('ascii', errors='replace')

    event_data.export_dll_name = dll_name or None

    timestamp = getattr(export_table.struct, 'TimeDateStamp', None)
    if timestamp:
      date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
      event_data.export_table_modification_time = date_time

  def _ParseImportTable(self, parser_mediator, pefile_object):
    """Parses the import table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      pefile_object (pefile.PE): pefile object.
    """
    import_table = getattr(pefile_object, 'DIRECTORY_ENTRY_IMPORT', None)
    if not import_table:
      return

    for table_entry in import_table:
      timestamp = getattr(table_entry.struct, 'TimeDateStamp', None)
      if not timestamp:
        continue

      dll_name = getattr(table_entry, 'dll', '')
      try:
        dll_name = dll_name.decode('ascii')
      except UnicodeDecodeError:
        dll_name = dll_name.decode('ascii', errors='replace')

      event_data = PEDLLImportEventData()
      event_data.delayed_import = False
      event_data.name = dll_name or None

      date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
      event_data.modification_time = date_time

      parser_mediator.ProduceEventData(event_data)

  def _ParseResourceSection(self, parser_mediator, pefile_object):
    """Parses the resource section.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      pefile_object (pefile.PE): pefile object.
    """
    resources = getattr(pefile_object, 'DIRECTORY_ENTRY_RESOURCE', None)
    if not resources:
      return

    message_table_resource = None
    winevt_template_resource = None
    for resource in resources.entries:
      if resource.name:
        resource_name = str(resource.name)
      else:
        resource_name = None

      timestamp = getattr(resource.directory, 'TimeDateStamp', None)
      if timestamp:
        event_data = PEResourceEventData()
        event_data.identifier = resource.id
        event_data.name = resource_name

        date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
        event_data.modification_time = date_time

        parser_mediator.ProduceEventData(event_data)

      # Only extract message strings from the first message table resource.
      if resource.id == 11 and not message_table_resource:
        message_table_resource = resource

      # Need to cast resource.name name to a string since it is of type
      # pefile.UnicodeStringWrapperPostProcessor and would fail the comparison
      # with 'WEVT_TEMPLATE' otherwise.
      elif resource_name == 'WEVT_TEMPLATE' and not winevt_template_resource:
        winevt_template_resource = resource

    if parser_mediator.extract_winevt_resources:
      # Only extract message strings from EventLog message files.
      message_file = parser_mediator.GetWindowsEventLogMessageFile()
      if message_file:
        parser_mediator.AddWindowsEventLogMessageFile(message_file)

        if message_table_resource:
          self._ParseMessageTableResource(
              parser_mediator, pefile_object, message_file,
              message_table_resource)

        if winevt_template_resource:
          self._ParseWevtTemplateResource(
              parser_mediator, pefile_object, message_file,
              winevt_template_resource)

  def _ParseMessageTable(
      self, parser_mediator, message_file, language_identifier, data):
    """Parses a message table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      message_file (WindowsEventLogMessageFileArtifact): Windows EventLog
          message file.
      language_identifier (int): language identifier (LCID).
      data (bytes): message table data.

    Raises:
      ParseError: when the message table cannot be parsed.
    """
    message_file_identifier = message_file.GetIdentifier()

    message_table_header_map = self._GetDataTypeMap('message_table_header')
    message_table_entry_map = self._GetDataTypeMap('message_table_entry')
    message_table_string_map = self._GetDataTypeMap('message_table_string')

    data_offset = 0

    context = dtfabric_data_maps.DataTypeMapContext()

    try:
      message_table_header = self._ReadStructureFromByteStream(
          data, data_offset, message_table_header_map, context=context)
    except (ValueError, errors.ParseError) as exception:
      raise errors.ParseError(
          'Unable to read message table header with error: {0!s}'.format(
              exception))

    data_offset += context.byte_size

    for entry_index in range(message_table_header.number_of_entries):
      context = dtfabric_data_maps.DataTypeMapContext()

      try:
        message_table_entry = self._ReadStructureFromByteStream(
            data[data_offset:], data_offset, message_table_entry_map,
            context=context)
      except (ValueError, errors.ParseError) as exception:
        raise errors.ParseError((
            'Unable to read message table entry: {0:d} at offset: {1:d} with '
            'error: {2!s}').format(entry_index, data_offset, exception))

      data_offset += context.byte_size

      message_identifier = message_table_entry.first_message_identifier
      string_offset = message_table_entry.first_string_offset
      while message_identifier <= message_table_entry.last_message_identifier:
        try:
          message_table_string = self._ReadStructureFromByteStream(
              data[string_offset:], string_offset, message_table_string_map)
        except (ValueError, errors.ParseError) as exception:
          raise errors.ParseError((
              'Unable to read message table string: 0x{0:08x} at offset: {1:d} '
              'with error: {2!s}').format(
                  message_identifier, string_offset, exception))

        if message_table_string.flags & 0x01:
          string_encoding = 'utf-16-le'
        else:
          string_encoding = parser_mediator.GetCodePage()

        try:
          string = message_table_string.data.decode(string_encoding)
        except UnicodeDecodeError:
          raise errors.ParseError((
              'Unable to decode {0:s} encoded message table string: 0x{1:08x} '
              'at offset: {2:d} with error: {3!s}').format(
                  string_encoding, message_identifier, string_offset,
                  exception))

        string_offset += message_table_string.data_size

        _, alignment_padding = divmod(string_offset, 4)
        if alignment_padding > 0:
          string_offset += 4 - alignment_padding

        string = self._resource_file_helper.FormatMessageStringInPEP3101(string)

        message_string = artifacts.WindowsEventLogMessageStringArtifact(
            language_identifier=language_identifier,
            message_identifier=message_identifier, string=string)
        message_string.SetMessageFileIdentifier(message_file_identifier)
        parser_mediator.AddWindowsEventLogMessageString(message_string)

        message_identifier += 1

  def _ParseMessageTableResource(
      self, parser_mediator, pefile_object, message_file,
      message_table_resource):
    """Parses a message table resource.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      pefile_object (pefile.PE): pefile object.
      message_file (WindowsEventLogMessageFileArtifact): Windows EventLog
          message file.
      message_table_resource (pefile.ResourceDirEntryData): message table
          resource.
    """
    if (not message_table_resource or not message_table_resource.directory or
        not message_table_resource.directory.entries or
        not message_table_resource.directory.entries[0].directory):
      return

    desired_language_tag = parser_mediator.GetLanguageTag().lower()

    for entry in message_table_resource.directory.entries[0].directory.entries:
      language_tag = languages.WindowsLanguageHelper.GetLanguageTagForLCID(
          entry.id)
      # TODO: add support for common language tag fallback.
      if not language_tag or language_tag.lower() != desired_language_tag:
        continue

      # TODO: use file offset?
      offset = getattr(entry.data.struct, 'OffsetToData', None)
      size = getattr(entry.data.struct, 'Size', None)
      data = pefile_object.get_memory_mapped_image()[offset:offset + size]

      self._ParseMessageTable(parser_mediator, message_file, entry.id, data)

  def _ParseWevtTemplate(self, parser_mediator, message_file, data):
    """Parses a WEVT_TEMPLATE.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      message_file (WindowsEventLogMessageFileArtifact): Windows EventLog
          message file.
      data (bytes): message table data.

    Raises:
      ParseError: when the message table cannot be parsed.
    """
    message_file_identifier = message_file.GetIdentifier()

    wevt_manifest_map = self._GetDataTypeMap('wevt_instrumentation_manifest')
    wevt_event_provider_map = self._GetDataTypeMap('wevt_event_provider')
    wevt_event_definitions_map = self._GetDataTypeMap('wevt_event_definitions')

    try:
      manifest = self._ReadStructureFromByteStream(
          data, 0, wevt_manifest_map)
    except (ValueError, errors.ParseError) as exception:
      raise errors.ParseError((
          'Unable to read WEVT instrumentation manifest with error: '
          '{0!s}').format(exception))

    for event_provider_descriptor in manifest.event_provider_descriptors:
      data_offset = event_provider_descriptor.data_offset
      try:
        event_provider = self._ReadStructureFromByteStream(
            data[data_offset:], data_offset, wevt_event_provider_map)
      except (ValueError, errors.ParseError) as exception:
        raise errors.ParseError(
            'Unable to read WEVT event provider with error: {0!s}'.format(
                exception))

      provider_identifier = '{{{0!s}}}'.format(
          event_provider_descriptor.provider_identifier)

      for provider_element_descriptor in (
          event_provider.provider_element_descriptors):
        data_offset = provider_element_descriptor.data_offset

        # Only extract event definitions.
        if data[data_offset:data_offset + 4] == b'EVNT':
          try:
            event_definitions = self._ReadStructureFromByteStream(
                data[data_offset:], data_offset, wevt_event_definitions_map)
          except (ValueError, errors.ParseError) as exception:
            raise errors.ParseError((
                'Unable to read WEVT event definitions with error: '
                '{0!s}').format(exception))
          for event_definition in event_definitions.definitions:
            event_definition = artifacts.WindowsWevtTemplateEvent(
                identifier=event_definition.identifier,
                message_identifier=event_definition.message_identifier,
                provider_identifier=provider_identifier,
                version=event_definition.version)
            event_definition.SetMessageFileIdentifier(message_file_identifier)

            parser_mediator.AddWindowsWevtTemplateEvent(event_definition)

  def _ParseWevtTemplateResource(
      self, parser_mediator, pefile_object, message_file,
      wevt_template_resource):
    """Parses a WEVT_TEMPLATE resource.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      pefile_object (pefile.PE): pefile object.
      message_file (WindowsEventLogMessageFileArtifact): Windows EventLog
          message file.
      wevt_template_resource (pefile.ResourceDirEntryData): WEVT_TEMPLATE
          resource.

    Raises:
      ParseError: when the message table cannot be parsed.
    """
    if (not wevt_template_resource or not wevt_template_resource.directory or
        not wevt_template_resource.directory.entries or
        not wevt_template_resource.directory.entries[0].directory):
      return

    desired_language_tag = parser_mediator.GetLanguageTag().lower()

    for entry in wevt_template_resource.directory.entries[0].directory.entries:
      language_tag = languages.WindowsLanguageHelper.GetLanguageTagForLCID(
          entry.id)
      # TODO: add support for common language tag fallback.
      if not language_tag or language_tag.lower() != desired_language_tag:
        continue

      # TODO: use file offset?
      offset = getattr(entry.data.struct, 'OffsetToData', None)
      size = getattr(entry.data.struct, 'Size', None)
      data = pefile_object.get_memory_mapped_image()[offset:offset + size]

      self._ParseWevtTemplate(parser_mediator, message_file, data)

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification."""
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(b'MZ', offset=0)
    return format_specification

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a Portable Executable (PE) file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      WrongParser: when the file cannot be parsed.
    """
    pe_data_slice = dfvfs_data_slice.DataSlice(file_object)
    try:
      pefile_object = pefile.PE(data=pe_data_slice, fast_load=True)
      pefile_object.parse_data_directories(directories=self._PE_DIRECTORIES)
    except Exception as exception:
      raise errors.WrongParser(
          'Unable to read PE file with error: {0!s}'.format(exception))

    event_data = PEFileEventData()
    # Note that the result of get_imphash() is an empty string if there is no
    # import hash.
    event_data.imphash = pefile_object.get_imphash() or None
    event_data.pe_type = self._GetPEType(pefile_object)
    event_data.section_names = self._GetSectionNames(pefile_object)

    timestamp = getattr(pefile_object.FILE_HEADER, 'TimeDateStamp', None)
    if timestamp:
      event_data.creation_time = dfdatetime_posix_time.PosixTime(
          timestamp=timestamp)

    self._ParseExportTable(pefile_object, event_data)

    self._ParseLoadConfigurationTable(pefile_object, event_data)

    parser_mediator.ProduceEventData(event_data)

    self._ParseImportTable(parser_mediator, pefile_object)

    self._ParseDelayImportTable(parser_mediator, pefile_object)

    self._ParseResourceSection(parser_mediator, pefile_object)


manager.ParsersManager.RegisterParser(PEParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the fake storage writer."""

import os
import unittest

from plaso.containers import events
from plaso.lib import definitions
from plaso.storage.sqlite import writer as sqlite_writer

from tests import test_lib as shared_test_lib
from tests.storage import test_lib
from tests.containers import test_lib as containers_test_lib


class SQLiteStorageFileWriterTest(test_lib.StorageTestCase):
  """Tests for the fake storage writer."""

  # pylint: disable=protected-access

  def _AddTestEvents(self, storage_writer):
    """Adds tests events to the storage writer.

    Args:
      storage_writer (SQLiteStorageFileWriter): storage writer.

    Returns:
      list[EventObject]: test events.
    """
    test_events = []
    for event, event_data, event_data_stream in (
        containers_test_lib.CreateEventsFromValues(self._TEST_EVENTS)):
      storage_writer.AddAttributeContainer(event_data_stream)

      event_data.SetEventDataStreamIdentifier(event_data_stream.GetIdentifier())
      storage_writer.AddAttributeContainer(event_data)

      event.SetEventDataIdentifier(event_data.GetIdentifier())
      storage_writer.AddAttributeContainer(event)

      test_events.append(event)

    return test_events

  def testAddAttributeContainer(self):
    """Tests the AddAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      storage_writer = sqlite_writer.SQLiteStorageFileWriter()
      storage_writer.Open(path=test_path)

      try:
        number_of_containers = storage_writer.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 0)

        storage_writer.AddAttributeContainer(event_data_stream)

        number_of_containers = storage_writer.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

      finally:
        storage_writer.Close()

      with self.assertRaises(IOError):
        storage_writer.AddAttributeContainer(event_data_stream)

  def testAddOrUpdateEventTag(self):
    """Tests the AddOrUpdateEventTag function."""
    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      storage_writer = sqlite_writer.SQLiteStorageFileWriter()
      storage_writer.Open(path=test_path)

      try:
        test_events = self._AddTestEvents(storage_writer)

        event_tag = events.EventTag()
        event_identifier = test_events[1].GetIdentifier()
        event_tag.SetEventIdentifier(event_identifier)

        event_tag.AddLabel('Label1')

        number_of_containers = storage_writer.GetNumberOfAttributeContainers(
            event_tag.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 0)

        storage_writer.AddOrUpdateEventTag(event_tag)

        number_of_containers = storage_writer.GetNumberOfAttributeContainers(
            event_tag.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

        event_tag = events.EventTag()
        event_identifier = test_events[2].GetIdentifier()
        event_tag.SetEventIdentifier(event_identifier)

        event_tag.AddLabel('Label2')

        storage_writer.AddOrUpdateEventTag(event_tag)

        number_of_containers = storage_writer.GetNumberOfAttributeContainers(
            event_tag.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 2)

        event_tag = events.EventTag()
        event_identifier = test_events[1].GetIdentifier()
        event_tag.SetEventIdentifier(event_identifier)

        event_tag.AddLabel('AnotherLabel1')

        storage_writer.AddOrUpdateEventTag(event_tag)

        number_of_containers = storage_writer.GetNumberOfAttributeContainers(
            event_tag.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 2)

        event_tags = list(storage_writer.GetAttributeContainers(
            event_tag.CONTAINER_TYPE))
        self.assertEqual(event_tags[0].labels, ['Label1', 'AnotherLabel1'])
        self.assertEqual(event_tags[1].labels, ['Label2'])

      finally:
        storage_writer.Close()

  # TODO: add tests for GetFirstWrittenEventSource
  # TODO: add tests for GetNextWrittenEventSource

  def testGetSortedEvents(self):
    """Tests the GetSortedEvents function."""
    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      storage_writer = sqlite_writer.SQLiteStorageFileWriter()
      storage_writer.Open(path=test_path)

      try:
        self._AddTestEvents(storage_writer)

        test_events = list(storage_writer.GetSortedEvents())
        self.assertEqual(len(test_events), 4)

      finally:
        storage_writer.Close()

    # TODO: add test with time range.

  def testOpenClose(self):
    """Tests the Open and Close functions."""
    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      storage_writer = sqlite_writer.SQLiteStorageFileWriter()
      storage_writer.Open(path=test_path)
      storage_writer.Close()

      storage_writer.Open(path=test_path)
      storage_writer.Close()

      storage_writer = sqlite_writer.SQLiteStorageFileWriter(
          storage_type=definitions.STORAGE_TYPE_TASK)
      storage_writer.Open(path=test_path)
      storage_writer.Close()

      storage_writer.Open(path=test_path)

      with self.assertRaises(IOError):
        storage_writer.Open(path=test_path)

      storage_writer.Close()

      with self.assertRaises(IOError):
        storage_writer.Close()


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Windows shell item custom event formatter helpers."""

from plaso.formatters import interface
from plaso.formatters import manager


class ShellItemFileEntryNameFormatterHelper(
    interface.CustomEventFormatterHelper):
  """Windows shell item file entry formatter helper."""

  IDENTIFIER = 'shell_item_file_entry_name'

  def FormatEventValues(self, output_mediator, event_values):
    """Formats event values using the helper.

    Args:
      output_mediator (OutputMediator): output mediator.
      event_values (dict[str, object]): event values.
    """
    event_values['file_entry_name'] = event_values.get('long_name', None)
    if not event_values['file_entry_name']:
      event_values['file_entry_name'] = event_values.get('name', None)


manager.FormattersManager.RegisterEventFormatterHelper(
    ShellItemFileEntryNameFormatterHelper)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Viminfo text parser plugin."""

import unittest

from plaso.parsers.text_plugins import viminfo
from plaso.containers import warnings

from tests.parsers.text_plugins import test_lib


class ViminfoTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the Viminfo text parser plugin."""

  # pylint: disable=protected-access

  def testProcess(self):
    """Tests the Process function."""
    plugin = viminfo.VimInfoTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['.viminfo'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 10)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test command line history event data.
    expected_event_values = {
        'data_type': 'viminfo:history',
        'history_type': 'Command Line History',
        'history_value': 'e TEST',
        'item_number': 0,
        'recorded_time': '2009-02-13T23:31:30+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Test search string history event data.
    expected_event_values = {
        'data_type': 'viminfo:history',
        'history_type': 'Search String History',
        'history_value': '/test_search',
        'item_number': 0,
        'recorded_time': '2009-02-13T23:31:32+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)

    # Test register event data.
    expected_event_values = {
        'data_type': 'viminfo:history',
        'history_type': 'Register',
        'history_value': 'test register',
        'item_number': 0,
        'recorded_time': '2009-02-13T23:31:34+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 4)
    self.CheckEventData(event_data, expected_event_values)

    # Test file mark event data.
    expected_event_values = {
        'data_type': 'viminfo:history',
        'history_type': 'File mark',
        'filename': '~\\_vimrc',
        'item_number': 0,
        'recorded_time': '2009-02-13T23:31:36+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 6)
    self.CheckEventData(event_data, expected_event_values)

    # Test jump list event data.
    expected_event_values = {
        'data_type': 'viminfo:history',
        'history_type': 'Jumplist',
        'filename': '~\\_vimrc',
        'item_number': 0,
        'recorded_time': '2009-02-13T23:31:38+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 8)
    self.CheckEventData(event_data, expected_event_values)

    # The extraction warnings are due to the last 6 lines of the test file.
    # These are the "history marks" which are currently not parsed.
    extraction_warnings = list(storage_writer.GetAttributeContainers(
        warnings.ExtractionWarning.CONTAINER_TYPE))

    expected_message = 'unable to parse log line: 64 "> ~\\_vimrc"'

    self.assertEqual(extraction_warnings[0].message, expected_message)

    expected_message = (
        'unable to parse log line: 68 "> C:\\Program Files (x86)\\Vim\\.vimrc"')

    self.assertEqual(extraction_warnings[3].message, expected_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS Application firewall log file text parser plugin."""

import unittest

from plaso.parsers.text_plugins import macos_appfirewall

from tests.parsers.text_plugins import test_lib


class MacOSAppFirewallTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the MacOS Application firewall log file text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = macos_appfirewall.MacOSAppFirewallTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['appfirewall.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 47)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Note that added_time contains a date time delta.
    expected_event_values = {
        'action': 'Allow TCP LISTEN  (in:0 out:1)',
        'added_time': '0000-11-29T22:17:30+00:00',
        'agent': 'socketfilterfw[87]',
        'computer_name': 'DarkTemplar-2.local',
        'data_type': 'macos:appfirewall_log:entry',
        'process_name': 'Spotify',
        'status': 'Info'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 38)
    self.CheckEventData(event_data, expected_event_values)

    # Check repeated lines.
    expected_event_values['added_time'] = '0000-11-29T22:18:29+00:00'

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 39)
    self.CheckEventData(event_data, expected_event_values)

    # Check year change.
    expected_event_values = {
        'action': 'Allow TCP LISTEN  (in:0 out:1)',
        'added_time': '0001-01-01T01:13:23+00:00',
        'agent': 'socketfilterfw[87]',
        'computer_name': 'DarkTemplar-2.local',
        'data_type': 'macos:appfirewall_log:entry',
        'process_name': 'Notify',
        'status': 'Info'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 46)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the SQLite plugin interface."""

import unittest

from plaso.parsers.sqlite_plugins import interface

from tests.parsers.sqlite_plugins import test_lib


class TestSQLitePlugin(interface.SQLitePlugin):
  """SQLite plugin for testing purposes."""

  NAME = 'test'

  QUERIES = [(
      'SELECT Field1, Field2, Field3 FROM MyTable', 'ParseMyTableRow')]

  REQUIRED_STRUCTURE = {
      'MyTable': frozenset(['Field1'])}

  SCHEMAS = [
      {'MyTable': (
          'CREATE TABLE "MyTable" ( `Field1` TEXT, `Field2` INTEGER, '
          '`Field3` BLOB )')}]

  def __init__(self):
    """Initializes a SQLite plugin."""
    super(TestSQLitePlugin, self).__init__()
    self.results = []

  # pylint: disable=unused-argument
  def ParseMyTableRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a MyTable row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    field1 = self._GetRowValue(query_hash, row, 'Field1')
    field2 = self._GetRowValue(query_hash, row, 'Field2')
    field3 = self._GetRowValue(query_hash, row, 'Field3')

    self.results.append((field1, field2, field3))


class SQLiteInterfaceTest(test_lib.SQLitePluginTestCase):
  """Tests for the SQLite plugin interface."""

  def testProcessWithWAL(self):
    """Tests the Process function on a database with WAL file."""
    plugin = TestSQLitePlugin()
    self._ParseDatabaseFileWithPlugin(
        ['wal_database.db'], plugin, wal_path_segments=['wal_database.db-wal'])

    expected_results = [
        ('Committed Text 1', 1, None),
        ('Committed Text 2', 2, None),
        ('Modified Committed Text 3', 4, None),
        ('Committed Text 4', 5, None),
        ('Committed Text 5', 7, None),
        ('Committed Text 6', 8, None),
        ('Committed Text 7', 9, None),
        ('Unhashable Row 1', 10, b'Binary Text!\x01\x02\x03'),
        ('Unhashable Row 2', 11, b'More Binary Text!\x01\x02\x03'),
        ('New Text 1', 12, None),
        ('New Text 2', 13, None)]

    self.assertEqual(plugin.results, expected_results)

  def testProcessWithoutWAL(self):
    """Tests the Process function on a database without WAL file."""
    plugin = TestSQLitePlugin()
    self._ParseDatabaseFileWithPlugin(['wal_database.db'], plugin)

    expected_results = [
        ('Committed Text 1', 1, None),
        ('Committed Text 2', 2, None),
        ('Deleted Text 1', 3, None),
        ('Committed Text 3', 4, None),
        ('Committed Text 4', 5, None),
        ('Deleted Text 2', 6, None),
        ('Committed Text 5', 7, None),
        ('Committed Text 6', 8, None),
        ('Committed Text 7', 9, None),
        ('Unhashable Row 1', 10, b'Binary Text!\x01\x02\x03')]

    self.assertEqual(plugin.results, expected_results)

  def testCheckSchema(self):
    """Tests the CheckSchema function."""
    plugin = TestSQLitePlugin()

    # Test matching schema.
    _, database = self._OpenDatabaseFile(['wal_database.db'])

    schema_match = plugin.CheckSchema(database)
    self.assertTrue(schema_match)

    # Test schema change with WAL.
    _, database = self._OpenDatabaseFile(
        ['wal_database.db'], wal_path_segments=['wal_database.db-wal'])

    schema_match = plugin.CheckSchema(database)
    self.assertFalse(schema_match)

    # Add schema change from WAL file and test again.
    plugin.SCHEMAS.append(
        {'MyTable':
         'CREATE TABLE "MyTable" ( `Field1` TEXT, `Field2` INTEGER, `Field3` '
         'BLOB , NewField TEXT)',
         'NewTable':
         'CREATE TABLE NewTable(NewTableField1 TEXT, NewTableField2 TEXT)'})

    schema_match = plugin.CheckSchema(database)
    self.assertTrue(schema_match)

    # Test without original schema.
    del plugin.SCHEMAS[0]

    schema_match = plugin.CheckSchema(database)
    self.assertTrue(schema_match)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Plist parser plugin for Safari Downloads.plist files."""

from plaso.containers import events
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


class SafariDownloadEventData(events.EventData):
  """Safari download event data.

  Attributes:
    end_time (dfdatetime.DateTimeValues): date and time the download was
        finished.
    full_path (str): full path where the file was downloaded to.
    received_bytes (int): number of bytes received while downloading.
    remove_on_completion (bool): remove the download when completed (done).
    start_time (dfdatetime.DateTimeValues): date and time the download was
        started.
    total_bytes (int): total number of bytes to download.
    url (str): URL of the downloaded file.
  """

  DATA_TYPE = 'safari:downloads:entry'

  def __init__(self):
    """Initializes event data."""
    super(SafariDownloadEventData, self).__init__(data_type=self.DATA_TYPE)
    self.end_time = None
    self.full_path = None
    self.received_bytes = None
    self.remove_on_completion = None
    self.start_time = None
    self.total_bytes = None
    self.url = None


class SafariDownloadsPlistPlugin(interface.PlistPlugin):
  """Plist parser plugin for Safari Downloads.plist files."""

  NAME = 'safari_downloads'
  DATA_FORMAT = 'Safari Downloads plist file'

  PLIST_PATH_FILTERS = frozenset([
      interface.PlistPathFilter('Downloads.plist')])

  PLIST_KEYS = frozenset(['DownloadHistory'])

  # pylint: disable=arguments-differ
  def _ParsePlist(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    for plist_key in match.get('DownloadHistory', []):
      event_data = SafariDownloadEventData()
      event_data.end_time = self._GetDateTimeValueFromPlistKey(
          plist_key, 'DownloadEntryDateFinishedKey')
      event_data.full_path = plist_key.get('DownloadEntryPath', None)
      event_data.received_bytes = plist_key.get(
          'DownloadEntryProgressBytesSoFar', None)
      event_data.remove_on_completion = plist_key.get(
          'DownloadEntryRemoveWhenDoneKey', None)
      event_data.start_time = self._GetDateTimeValueFromPlistKey(
          plist_key, 'DownloadEntryDateAddedKey')
      event_data.total_bytes = plist_key.get(
          'DownloadEntryProgressTotalToLoad', None)
      event_data.url = plist_key.get('DownloadEntryURL', None)

      parser_mediator.ProduceEventData(event_data)


plist.PlistParser.RegisterPlugin(SafariDownloadsPlistPlugin)



# -*- coding: utf-8 -*-
"""Output module that saves data into a JSON format."""

import json

from plaso.output import manager
from plaso.output import shared_json


class JSONOutputModule(shared_json.SharedJSONOutputModule):
  """Output module for the JSON format."""

  NAME = 'json'
  DESCRIPTION = 'Saves the events into a JSON format.'

  def __init__(self):
    """Initializes an output module."""
    super(JSONOutputModule, self).__init__()
    self._event_counter = 0

  def _WriteFieldValues(self, output_mediator, field_values):
    """Writes field values to the output.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      field_values (dict[str, str]): output field values per name.
    """
    if self._event_counter != 0:
      self.WriteText(', ')

    json_string = json.dumps(field_values, sort_keys=True)
    output_text = '"event_{0:d}": {1:s}\n'.format(
        self._event_counter, json_string)
    self.WriteText(output_text)

    self._event_counter += 1

  def WriteFooter(self):
    """Writes the footer to the output."""
    self.WriteText('}')

  def WriteHeader(self, output_mediator):
    """Writes the header to the output.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
    """
    self.WriteText('{')
    self._event_counter = 0


manager.OutputManager.RegisterOutput(JSONOutputModule)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Opera browser history parsers."""

import unittest

from plaso.parsers import opera

from tests.parsers import test_lib


class OperaTypedParserTest(test_lib.ParserTestCase):
  """Tests for the Opera Typed History parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = opera.OperaTypedHistoryParser()
    storage_writer = self._ParseFile(['typed_history.xml'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'opera:history:typed_entry',
        'entry_selection': 'Filled from autocomplete.',
        'last_typed_time': '2013-11-11T23:45:27+00:00',
        'url': 'plaso.kiddaland.net'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


class OperaGlobalParserTest(test_lib.ParserTestCase):
  """Tests for the Opera Global History parser."""

  def testParseFile(self):
    """Read a history file and run a few tests."""
    parser = opera.OperaGlobalHistoryParser()
    storage_writer = self._ParseFile(['global_history.dat'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 37)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'opera:history:entry',
        'description': 'First and Only Visit',
        'last_visited_time': '2013-11-11T22:45:46+00:00',
        'title': 'Karl Bretaprins fær ellilífeyri - mbl.is',
        'url': (
            'http://www.mbl.is/frettir/erlent/2013/11/11/'
            'karl_bretaprins_faer_ellilifeyri/')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 4)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows IIS log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import iis

from tests.parsers.text_plugins import test_lib


class WinIISTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the Windows IIS text parser plugin."""

  def testProcessWithIIS6Log(self):
    """Tests the Process function with an IIS 6 log file."""
    plugin = iis.WinIISTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['iis6.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 12)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'iis:log:line',
        'dest_ip': '10.10.10.100',
        'dest_port': 80,
        'http_method': 'GET',
        'http_status': 200,
        'last_written_time': '2013-07-30T00:00:00+00:00',
        'requested_uri_stem': '/some/image/path/something.jpg',
        'source_ip': '10.10.10.100',
        'user_agent': (
            'Mozilla/4.0+(compatible;+Win32;+WinHttp.WinHttpRequest.5)')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithIIS7LogAndSQLI(self):
    """Tests the Process function with an IIS 7 log file with SQLI."""
    plugin = iis.WinIISTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['iis7_sqli.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'iis:log:line',
        'dest_ip': '111.111.111.111',
        'dest_port': 443,
        'http_method': 'GET',
        'http_status': 500,
        'last_written_time': '2015-10-16T13:01:02+00:00',
        'requested_uri_stem': '/foo/bar/baz.asp',
        'source_ip': '222.222.222.222',
        'user_agent': (
            'Mozilla/5.0+(Macintosh;+Intel+Mac+OS+X+10_9_2)+AppleWebKit/'
            '537.36+(KHTML,+like+Gecko)+Chrome/34.0.1847.131+Safari/537.36')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithIIS7OWALog(self):
    """Tests the Process function with an IIS 7 OWA log file."""
    plugin = iis.WinIISTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['iis7_owa.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'iis:log:line',
        'dest_ip': '10.11.2.3',
        'dest_port': 443,
        'http_method': 'GET',
        'http_status': 200,
        'last_written_time': '2015-12-31T00:19:48+00:00',
        'requested_uri_stem': '/owa/',
        'source_ip': '77.123.22.98',
        'user_agent': (
            'Mozilla/5.0+(Windows+NT+6.1;+WOW64)+AppleWebKit/537.36+'
            '(KHTML,+like+Gecko)+Chrome/39.0.2171.95+Safari/537.36')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithoutDate(self):
    """Tests the Process function with logs without a date column."""
    plugin = iis.WinIISTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['iis_without_date.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 11)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'iis:log:line',
        'last_written_time': '2013-07-30T00:00:03+00:00',
        'protocol_version': 'HTTP/1.1'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithIIS10Log(self):
    """Tests the Process function with an IIS 10 log file."""
    plugin = iis.WinIISTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['iis10.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'iis:log:line',
        'dest_ip': '111.111.111.111',
        'dest_port': 80,
        'http_method': 'GET',
        'http_status': 200,
        'last_written_time': '2021-04-01T00:00:21+00:00',
        'requested_uri_stem': '/foo/bar/baz.asp',
        'source_ip': '222.222.222.222',
        'user_agent': (
            'Mozilla/5.0+(Windows+NT+5.1)+AppleWebKit/'
            '537.36+(KHTML,+like+Gecko)'
            '+Chrome/35.0.2309.372+Safari/537.36')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    plugin = iis.WinIISTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['iis10_edge_cases.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 12)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite parser plugin for Google Chrome cookies database files."""

from dfdatetime import webkit_time as dfdatetime_webkit_time

from plaso.containers import events
from plaso.lib import cookie_plugins_helper
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class ChromeCookieEventData(events.EventData):
  """Chrome Cookie event data.

  Attributes:
    access_time (dfdatetime.DateTimeValues): date and time the cookie
        was last accessed.
    cookie_name (str): name of the cookie.
    creation_time (dfdatetime.DateTimeValues): date and time the cookie
        was created.
    data (str): value of the cookie.
    expiration_time (dfdatetime.DateTimeValues): date and time the cookie
        expires.
    host (str): hostname of host that set the cookie value.
    httponly (bool): True if the cookie cannot be accessed through client
        side script.
    path (str): path where the cookie got set.
    persistent (bool): True if the cookie is persistent.
    query (str): SQL query that was used to obtain the event data.
    secure (bool): True if the cookie should only be transmitted over a
        secure channel.
    url (str): URL or path where the cookie got set.
  """

  DATA_TYPE = 'chrome:cookie:entry'

  def __init__(self):
    """Initializes event data."""
    super(ChromeCookieEventData, self).__init__(data_type=self.DATA_TYPE)
    self.access_time = None
    self.cookie_name = None
    self.creation_time = None
    self.data = None
    self.expiration_time = None
    self.host = None
    self.httponly = None
    self.path = None
    self.persistent = None
    self.query = None
    self.secure = None
    self.url = None


class BaseChromeCookiePlugin(
    interface.SQLitePlugin, cookie_plugins_helper.CookiePluginsHelper):
  """SQLite parser plugin for Google Chrome cookies database files."""

  # Google Analytics __utmz variable translation.
  GA_UTMZ_TRANSLATION = {
      'utmcsr': 'Last source used to access.',
      'utmccn': 'Ad campaign information.',
      'utmcmd': 'Last type of visit.',
      'utmctr': 'Keywords used to find site.',
      'utmcct': 'Path to the page of referring link.'}

  def _GetDateTimeRowValue(self, query_hash, row, value_name):
    """Retrieves a date and time value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.WebKitTime: date and time value or None if not available.
    """
    timestamp = self._GetRowValue(query_hash, row, value_name)
    if timestamp is None:
      return None

    return dfdatetime_webkit_time.WebKitTime(timestamp=timestamp)

  def ParseCookieRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a cookie row.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      query (str): query that created the row.
      row (sqlite3.Row): row resulting from the query.
    """
    query_hash = hash(query)

    cookie_name = self._GetRowValue(query_hash, row, 'name')
    cookie_data = self._GetRowValue(query_hash, row, 'value')

    hostname = self._GetRowValue(query_hash, row, 'host_key')
    if hostname.startswith('.'):
      hostname = hostname[1:]

    httponly = self._GetRowValue(query_hash, row, 'httponly')
    path = self._GetRowValue(query_hash, row, 'path')
    persistent = self._GetRowValue(query_hash, row, 'persistent')
    secure = self._GetRowValue(query_hash, row, 'secure')

    if secure:
      scheme = 'https'
    else:
      scheme = 'http'

    url = '{0:s}://{1:s}{2:s}'.format(scheme, hostname, path)

    event_data = ChromeCookieEventData()
    event_data.access_time = self._GetDateTimeRowValue(
        query_hash, row, 'last_access_utc')
    event_data.cookie_name = cookie_name
    event_data.creation_time = self._GetDateTimeRowValue(
        query_hash, row, 'creation_utc')
    event_data.data = cookie_data
    event_data.expiration_time = self._GetDateTimeRowValue(
        query_hash, row, 'expires_utc')
    event_data.host = hostname
    event_data.httponly = bool(httponly)
    event_data.path = path
    event_data.persistent = bool(persistent)
    event_data.query = query
    event_data.secure = bool(secure)
    event_data.url = url

    parser_mediator.ProduceEventData(event_data)

    self._ParseCookie(parser_mediator, cookie_name, cookie_data, url)


class Chrome17CookiePlugin(BaseChromeCookiePlugin):
  """SQLite parser plugin for Google Chrome 17 - 65 cookies database files."""

  NAME = 'chrome_17_cookies'
  DATA_FORMAT = 'Google Chrome 17 - 65 cookies SQLite database file'

  REQUIRED_STRUCTURE = {
      'cookies': frozenset([
          'creation_utc', 'host_key', 'name', 'value', 'path', 'expires_utc',
          'secure', 'httponly', 'last_access_utc', 'has_expires',
          'persistent']),
      'meta': frozenset([])}

  QUERIES = [
      (('SELECT creation_utc, host_key, name, value, path, expires_utc, '
        'secure, httponly, last_access_utc, has_expires, persistent '
        'FROM cookies'), 'ParseCookieRow')]

  SCHEMAS = [{
      'cookies': (
          'CREATE TABLE cookies (creation_utc INTEGER NOT NULL UNIQUE PRIMARY '
          'KEY, host_key TEXT NOT NULL, name TEXT NOT NULL, value TEXT NOT '
          'NULL, path TEXT NOT NULL, expires_utc INTEGER NOT NULL, secure '
          'INTEGER NOT NULL, httponly INTEGER NOT NULL, last_access_utc '
          'INTEGER NOT NULL, has_expires INTEGER DEFAULT 1, persistent '
          'INTEGER DEFAULT 1)'),
      'meta': (
          'CREATE TABLE meta(key LONGVARCHAR NOT NULL UNIQUE PRIMARY KEY, '
          'value LONGVARCHAR)')}]


class Chrome66CookiePlugin(BaseChromeCookiePlugin):
  """SQLite parser plugin for Google Chrome 66+ cookies database files."""

  NAME = 'chrome_66_cookies'
  DATA_FORMAT = 'Google Chrome 66 and later cookies SQLite database file'

  REQUIRED_STRUCTURE = {
      'cookies': frozenset([
          'creation_utc', 'host_key', 'name', 'value', 'path', 'expires_utc',
          'is_secure', 'is_httponly', 'last_access_utc', 'has_expires',
          'is_persistent']),
      'meta': frozenset([])}

  QUERIES = [
      (('SELECT creation_utc, host_key, name, value, path, expires_utc, '
        'is_secure AS secure, is_httponly AS httponly, last_access_utc, '
        'has_expires, is_persistent AS persistent '
        'FROM cookies'), 'ParseCookieRow')]

  SCHEMAS = [{
      'cookies': (
          'CREATE TABLE cookies (creation_utc INTEGER NOT NULL, host_key TEXT '
          'NOT NULL, name TEXT NOT NULL, value TEXT NOT NULL, path TEXT NOT '
          'NULL, expires_utc INTEGER NOT NULL, is_secure INTEGER NOT NULL, '
          'is_httponly INTEGER NOT NULL, last_access_utc INTEGER NOT NULL, '
          'has_expires INTEGER NOT NULL DEFAULT 1, is_persistent INTEGER NOT '
          'NULL DEFAULT 1, priority INTEGER NOT NULL DEFAULT 1, '
          'encrypted_value BLOB DEFAULT \'\', firstpartyonly INTEGER NOT NULL '
          'DEFAULT 0, UNIQUE (host_key, name, path))'),
      'meta': (
          'CREATE TABLE meta(key LONGVARCHAR NOT NULL UNIQUE PRIMARY KEY, '
          'value LONGVARCHAR)')}]


sqlite.SQLiteParser.RegisterPlugins([
    Chrome17CookiePlugin, Chrome66CookiePlugin])



# -*- coding: utf-8 -*-
"""Plist parser plugin for MacOS install history plist files."""

from plaso.containers import events
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


class MacOSInstallHistoryEventData(events.EventData):
  """MacOS install history event data.

  Attributes:
    identifiers (list[str]): identifiers of the installed package.
    name (str): display name of the installed package.
    process_name (str): name of the process that installed the package.
    version (str): display version of the installed package.
    written_time (dfdatetime.DateTimeValues): entry written date and time.
  """

  DATA_TYPE = 'macos:install_history:entry'

  def __init__(self):
    """Initializes event data."""
    super(MacOSInstallHistoryEventData, self).__init__(data_type=self.DATA_TYPE)
    self.identifiers = None
    self.name = None
    self.process_name = None
    self.version = None
    self.written_time = None


class MacOSInstallHistoryPlistPlugin(interface.PlistPlugin):
  """Plist parser plugin for MacOS install history plist files."""

  NAME = 'macosx_install_history'
  DATA_FORMAT = 'MacOS installation history plist file'

  PLIST_PATH_FILTERS = frozenset([
      interface.PlistPathFilter('InstallHistory.plist')])

  PLIST_KEYS = frozenset([
      'date', 'displayName', 'displayVersion', 'processName',
      'packageIdentifiers'])

  # pylint: disable=arguments-differ
  def _ParsePlist(self, parser_mediator, top_level=None, **unused_kwargs):
    """Extracts relevant install history entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      top_level (Optional[dict[str, object]]): plist top-level item.
    """
    for plist_key in top_level:
      event_data = MacOSInstallHistoryEventData()
      event_data.identifiers = plist_key.get('packageIdentifiers', None)
      event_data.name = plist_key.get('displayName', None)
      event_data.process_name = plist_key.get('processName', None)
      event_data.version = plist_key.get('displayVersion', None)
      event_data.written_time = self._GetDateTimeValueFromPlistKey(
          plist_key, 'date')

      parser_mediator.ProduceEventData(event_data)


plist.PlistParser.RegisterPlugin(MacOSInstallHistoryPlistPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the storage reader."""

import unittest

from acstore.containers import interface as containers_interface

from plaso.containers import event_sources
from plaso.storage import reader
from plaso.storage.fake import fake_store

from tests.storage import test_lib


class StorageReaderTest(test_lib.StorageTestCase):
  """Tests for the storage reader."""

  # pylint: disable=protected-access

  def testInitialization(self):
    """Tests the __init__ function."""
    test_reader = reader.StorageReader()
    self.assertIsNotNone(test_reader)

  # TODO: add tests for __enter__ and __exit__
  # TODO: add tests for Close

  def testGetAttributeContainerByIdentifier(self):
    """Tests the GetAttributeContainerByIdentifier function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()
    test_reader._store.Open()

    try:
      event_source = event_sources.EventSource()
      test_reader._store.AddAttributeContainer(event_source)

      test_identifier = event_source.GetIdentifier()
      test_container = test_reader.GetAttributeContainerByIdentifier(
          event_source.CONTAINER_TYPE, test_identifier)
      self.assertIsNotNone(test_container)

      test_identifier = containers_interface.AttributeContainerIdentifier(
          name=event_source.CONTAINER_TYPE, sequence_number=99)
      test_container = test_reader.GetAttributeContainerByIdentifier(
          event_source.CONTAINER_TYPE, test_identifier)
      self.assertIsNone(test_container)

    finally:
      test_reader._store.Close()

  def testGetAttributeContainerByIndex(self):
    """Tests the GetAttributeContainerByIndex function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()
    test_reader._store.Open()

    try:
      event_source = event_sources.EventSource()
      test_reader._store.AddAttributeContainer(event_source)

      test_container = test_reader.GetAttributeContainerByIndex(
          event_source.CONTAINER_TYPE, 0)
      self.assertIsNotNone(test_container)

      test_container = test_reader.GetAttributeContainerByIndex(
          event_source.CONTAINER_TYPE, 99)
      self.assertIsNone(test_container)

    finally:
      test_reader._store.Close()

  def testGetAttributeContainers(self):
    """Tests the GetAttributeContainers function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()
    test_reader._store.Open()

    try:
      event_source = event_sources.EventSource()
      test_reader._store.AddAttributeContainer(event_source)

      test_generator = test_reader.GetAttributeContainers(
          event_source.CONTAINER_TYPE)
      test_containers = list(test_generator)
      self.assertEqual(len(test_containers), 1)

    finally:
      test_reader._store.Close()

  def testGetFormatVersion(self):
    """Tests the GetFormatVersion function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()

    format_version = test_reader.GetFormatVersion()
    self.assertIsNone(format_version)

  def testGetNumberOfAttributeContainers(self):
    """Tests the GetNumberOfAttributeContainers function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()
    test_reader._store.Open()

    try:
      event_source = event_sources.EventSource()
      test_reader._store.AddAttributeContainer(event_source)

      number_of_containers = test_reader.GetNumberOfAttributeContainers(
          event_source.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

    finally:
      test_reader._store.Close()

  def testGetSerializationFormat(self):
    """Tests the GetSerializationFormat function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()

    serialization_format = test_reader.GetSerializationFormat()
    self.assertIsNone(serialization_format)

  # TODO: add tests for GetSessions
  # TODO: add tests for GetSortedEvents

  def testHasAttributeContainers(self):
    """Tests the HasAttributeContainers function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()
    test_reader._store.Open()

    try:
      event_source = event_sources.EventSource()
      test_reader._store.AddAttributeContainer(event_source)

      result = test_reader.HasAttributeContainers(event_source.CONTAINER_TYPE)
      self.assertTrue(result)

    finally:
      test_reader._store.Close()

  def testSetSerializersProfiler(self):
    """Tests the SetSerializersProfiler function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()

    test_reader.SetSerializersProfiler(None)

  def testSetStorageProfiler(self):
    """Tests the SetStorageProfiler function."""
    test_reader = reader.StorageReader()
    test_reader._store = fake_store.FakeStore()

    test_reader.SetStorageProfiler(None)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite parser plugin for Mozilla Firefox downloads database files."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class FirefoxDownloadEventData(events.EventData):
  """Firefox download event data.

  Attributes:
    end_time (dfdatetime.DateTimeValues): date and time the download was
        finished.
    full_path (str): full path of the target of the download.
    mime_type (str): mime type of the download.
    name (str): name of the download.
    offset (str): identifier of the row, from which the event data was
        extracted.
    query (str): SQL query that was used to obtain the event data.
    received_bytes (int): number of bytes received.
    referrer (str): referrer URL of the download.
    start_time (dfdatetime.DateTimeValues): date and time the download was
        started.
    temporary_location (str): temporary location of the download.
    total_bytes (int): total number of bytes of the download.
    url (str): source URL of the download.
  """

  DATA_TYPE = 'firefox:downloads:download'

  def __init__(self):
    """Initializes event data."""
    super(FirefoxDownloadEventData, self).__init__(data_type=self.DATA_TYPE)
    self.end_time = None
    self.full_path = None
    self.mime_type = None
    self.name = None
    self.offset = None
    self.query = None
    self.received_bytes = None
    self.referrer = None
    self.start_time = None
    self.temporary_location = None
    self.total_bytes = None
    self.url = None


class FirefoxDownloadsPlugin(interface.SQLitePlugin):
  """SQLite parser plugin for Mozilla Firefox downloads database files.

  The Mozilla Firefox downloads database file is typically stored in:
  downloads.sqlite
  """

  NAME = 'firefox_downloads'
  DATA_FORMAT = (
      'Mozilla Firefox downloads SQLite database (downloads.sqlite) file')

  REQUIRED_STRUCTURE = {
      'moz_downloads': frozenset([
          'id', 'name', 'source', 'target', 'tempPath', 'startTime', 'endTime',
          'state', 'referrer', 'currBytes', 'maxBytes', 'mimeType'])}

  QUERIES = [
      (('SELECT moz_downloads.id, moz_downloads.name, moz_downloads.source, '
        'moz_downloads.target, moz_downloads.tempPath, '
        'moz_downloads.startTime, moz_downloads.endTime, moz_downloads.state, '
        'moz_downloads.referrer, moz_downloads.currBytes, '
        'moz_downloads.maxBytes, moz_downloads.mimeType '
        'FROM moz_downloads'),
       'ParseDownloadsRow')]

  SCHEMAS = [
      {'moz_downloads':
       'CREATE TABLE moz_downloads (id INTEGER PRIMARY KEY, name TEXT, '
       'source TEXT, target TEXT, tempPath TEXT, startTime INTEGER, endTime '
       'INTEGER, state INTEGER, referrer TEXT, entityID TEXT, currBytes '
       'INTEGER NOT NULL DEFAULT 0, maxBytes INTEGER NOT NULL DEFAULT -1, '
       'mimeType TEXT, preferredApplication TEXT, preferredAction INTEGER '
       'NOT NULL DEFAULT 0, autoResume INTEGER NOT NULL DEFAULT 0)'}]

  def _GetDateTimeRowValue(self, query_hash, row, value_name):
    """Retrieves a date and time value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.PosixTimeInMicroseconds: date and time value or None if not
          available.
    """
    timestamp = self._GetRowValue(query_hash, row, value_name)
    if timestamp is None:
      return None

    return dfdatetime_posix_time.PosixTimeInMicroseconds(timestamp=timestamp)

  def ParseDownloadsRow(
      self, parser_mediator, query, row, **unused_kwargs):
    """Parses a downloads row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = FirefoxDownloadEventData()
    event_data.end_time = self._GetDateTimeRowValue(query_hash, row, 'endTime')
    event_data.full_path = self._GetRowValue(query_hash, row, 'target')
    event_data.mime_type = self._GetRowValue(query_hash, row, 'mimeType')
    event_data.name = self._GetRowValue(query_hash, row, 'name')
    event_data.offset = self._GetRowValue(query_hash, row, 'id')
    event_data.query = query
    event_data.received_bytes = self._GetRowValue(query_hash, row, 'currBytes')
    event_data.referrer = self._GetRowValue(query_hash, row, 'referrer')
    event_data.start_time = self._GetDateTimeRowValue(
        query_hash, row, 'startTime')
    event_data.temporary_location = self._GetRowValue(
        query_hash, row, 'tempPath')
    event_data.total_bytes = self._GetRowValue(query_hash, row, 'maxBytes')
    event_data.url = self._GetRowValue(query_hash, row, 'source')

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(FirefoxDownloadsPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows Defender History parser."""

import unittest

from plaso.parsers import windefender_history

from tests.parsers import test_lib


class WinDefenderHistoryUnitTest(test_lib.ParserTestCase):
  """Tests for the Windows Defender History parser."""

  def testWebFileDetection(self):
    """Tests parsing a webfile Detection History file."""
    parser = windefender_history.WinDefenderHistoryParser()
    storage_writer = self._ParseFile([
        'FC380697-A68D-4C94-B67F-9B6449039463'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'av:defender:detection_history',
        'filename': 'C:\\Users\\testuser\\Downloads\\PotentiallyUnwanted.exe',
        'host_and_user': 'FRYPTOP\\testuser',
        'process': 'C:\\Windows\\explorer.exe',
        'recorded_time': '2022-07-22T05:32:27.8302783+00:00',
        'sha256': (
            'd6f6c6b9fde37694e12b12009ad11ab9ec8dd0f193e7319c523933bdad8a50ad'),
        'threat_name': 'PUA:Win32/EICAR_Test_File',
        'web_filenames': [(
            'C:\\Users\\testuser\\Downloads\\PotentiallyUnwanted.exe|'
            'http://amtso.eicar.org/PotentiallyUnwanted.exe|'
            'pid:19588,ProcessStart:133029415464710822')]}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testContainerDetection(self):
    """Tests parsing a containerfile Detection History file."""
    parser = windefender_history.WinDefenderHistoryParser()
    storage_writer = self._ParseFile([
        '6AFE33A0-19BA-4FFF-892F-B700539D7D63'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'container_filenames': [
            'C:\\Users\\testuser\\Downloads\\eicar_com.zip'],
        'data_type': 'av:defender:detection_history',
        'filename': 'C:\\Users\\testuser\\Downloads\\eicar_com.zip->eicar.com',
        'host_and_user': 'FRYPTOP\\testuser',
        'process': 'Unknown',
        'recorded_time': '2022-07-22T05:36:18.9094421+00:00',
        'sha256': (
            '275a021bbfb6489e54d471899f7db9d1663fc695ec2fe2a2c4538aabf651fd0f'),
        'threat_name': 'Virus:DOS/EICAR_Test_File',
        'web_filenames': [(
            'C:\\Users\\testuser\\Downloads\\eicar_com.zip|'
            'https://secure.eicar.org/eicar_com.zip|'
            'pid:20580,ProcessStart:133029417782902463')]}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the SQLite parser plugin for iOS Screen Time database files."""

import unittest

from plaso.parsers.sqlite_plugins import ios_screentime

from tests.parsers.sqlite_plugins import test_lib


class IOSScreenTimePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the SQLite parser plugin for iOS Screen Time database files."""

  def testParse(self):
    """Tests the ParseScreenTimeRow method."""
    plugin = ios_screentime.IOSScreenTimePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['ios_screentime.sqlite'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 777)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'bundle_identifier': 'com.spotify.client',
        'device_identifier': 'c623fbd7e91b041e07a68f8523f53a35973e475d',
        'device_name': 'iPhone',
        'domain': None,
        'start_time': '2021-02-03T16:00:00.000000+00:00',
        'total_time': 1006,
        'user_family_name': 'DFIR',
        'user_given_name': 'This Is'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Shared functionality for JSON based output modules."""

import abc

from acstore.containers import interface as containers_interface

from dfdatetime import interface as dfdatetime_interface

from plaso.output import formatting_helper
from plaso.output import text_file
from plaso.serializer import json_serializer


class JSONFieldFormattingHelper(formatting_helper.FieldFormattingHelper):
  """JSON output module field formatting helper."""

  # Maps the name of a fields to a a callback function that formats
  # the field value.
  _FIELD_FORMAT_CALLBACKS = {
      'display_name': '_FormatDisplayName',
      'filename': '_FormatFilename',
      'inode': '_FormatInode',
      'message': '_FormatMessage',
      'values': '_FormatValues'}

  # The field format callback methods require specific arguments hence
  # the check for unused arguments is disabled here.
  # pylint: disable=unused-argument

  def _FormatValues(
      self, output_mediator, event, event_data, event_data_stream):
    """Formats a values.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.

    Returns:
      list[dict[str, str]]: values field.
    """
    values = event_data.values
    if isinstance(values, list) and event_data.data_type in (
        'windows:registry:key_value', 'windows:registry:service'):
      values = [
          {'data': data, 'data_type': data_type, 'name': name}
          for name, data_type, data in sorted(values)]

    return values

  # pylint: enable=unused-argument

  def GetFormattedField(
      self, output_mediator, field_name, event, event_data, event_data_stream,
      event_tag):
    """Formats the specified field.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      field_name (str): name of the field.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
      event_tag (EventTag): event tag.

    Returns:
      object: value of the field or None if not available.
    """
    if field_name in self._event_tag_field_names:
      return self._FormatTag(output_mediator, event_tag)

    callback_function = self._callback_functions.get(field_name, None)
    if callback_function:
      output_value = callback_function(
          output_mediator, event, event_data, event_data_stream)
    elif field_name in self._event_data_stream_field_names:
      output_value = getattr(event_data_stream, field_name, None)
    else:
      output_value = getattr(event_data, field_name, None)

    return output_value


class SharedJSONOutputModule(text_file.TextFileOutputModule):
  """Shared functionality for JSON based output modules."""

  _JSON_SERIALIZER = json_serializer.JSONAttributeContainerSerializer

  _GENERATED_FIELD_VALUES = ['display_name', 'filename', 'inode']

  def __init__(self):
    """Initializes an output module."""
    super(SharedJSONOutputModule, self).__init__()
    self._field_formatting_helper = JSONFieldFormattingHelper()

  def _GetFieldValues(
      self, output_mediator, event, event_data, event_data_stream, event_tag):
    """Retrieves the output field values.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
      event_tag (EventTag): event tag.

    Returns:
      dict[str, str]: output field values per name.
    """
    field_values = {
        '__container_type__': 'event',
        '__type__': 'AttributeContainer'}

    if event_data:
      for attribute_name, attribute_value in event_data.GetAttributes():
        # Ignore attribute container identifier and date and time values.
        if isinstance(attribute_value, (
            containers_interface.AttributeContainerIdentifier,
            dfdatetime_interface.DateTimeValues)):
          continue

        if (isinstance(attribute_value, list) and attribute_value and
            isinstance(attribute_value[0],
                       dfdatetime_interface.DateTimeValues)):
          continue

        # Output _parser_chain as parser for backwards compatibility.
        if attribute_name == '_parser_chain':
          attribute_name = 'parser'

        field_value = self._field_formatting_helper.GetFormattedField(
            output_mediator, attribute_name, event, event_data,
            event_data_stream, event_tag)
        field_values[attribute_name] = field_value

    if event_data_stream:
      for attribute_name, attribute_value in event_data_stream.GetAttributes():
        # Output path_spec as pathspec for backwards compatibility.
        if attribute_name == 'path_spec':
          attribute_name = 'pathspec'
          attribute_value = self._JSON_SERIALIZER.WriteSerializedDict(
              attribute_value)

        field_values[attribute_name] = attribute_value

    if event:
      for attribute_name, attribute_value in event.GetAttributes():
        # Ignore attribute container identifier values.
        if isinstance(attribute_value,
                      containers_interface.AttributeContainerIdentifier):
          continue

        if attribute_name == 'date_time':
          attribute_value = self._JSON_SERIALIZER.WriteSerializedDict(
              attribute_value)

        field_values[attribute_name] = attribute_value

    for field_name in self._GENERATED_FIELD_VALUES:
      field_value = field_values.get(field_name, None)
      if field_value is None:
        field_value = self._field_formatting_helper.GetFormattedField(
            output_mediator, field_name, event, event_data, event_data_stream,
            event_tag)
        field_values[field_name] = field_value

    field_values['message'] = self._field_formatting_helper.GetFormattedField(
        output_mediator, 'message', event, event_data, event_data_stream,
        event_tag)

    if event_tag:
      event_tag_values = {
          '__container_type__': 'event_tag',
          '__type__': 'AttributeContainer'}

      for attribute_name, attribute_value in event_tag.GetAttributes():
        # Ignore attribute container identifier values.
        if isinstance(attribute_value,
                      containers_interface.AttributeContainerIdentifier):
          continue

        event_tag_values[attribute_name] = attribute_value

      field_values['tag'] = event_tag_values

    return field_values

  @abc.abstractmethod
  def _WriteFieldValues(self, output_mediator, field_values):
    """Writes field values to the output.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      field_values (dict[str, str]): output field values per name.
    """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome extension activity database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import chrome_extension_activity

from tests.parsers.sqlite_plugins import test_lib


class ChromeExtensionActivityPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Chrome extension activity database plugin."""

  def testProcess(self):
    """Tests the Process function on a Chrome extension activity database."""
    plugin = chrome_extension_activity.ChromeExtensionActivityPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['Extension Activity'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 56)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'action_type': 1,
        'activity_id': 48,
        'api_name': 'browserAction.onClicked',
        'data_type': 'chrome:extension_activity:activity_log',
        'extension_id': 'ognampngfcbddbfemdapefohjiobgbdl',
        'recorded_time': '2014-11-25T21:08:23.698737+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()
